#pragma once
#ifndef __HEADER___COST_EMPTY_SAMPLE__
#define __HEADER___COST_EMPTY_SAMPLE__

#include<CGAL/squared_distance_2.h>
#include<CGAL/squared_distance_3.h>
#include<CGAL/Cartesian/line_constructions_2.h>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/ml/ml.hpp>
#include <opencv2/core.hpp>
#include <opencv2/imgproc.hpp>
#include "opencv2/imgcodecs.hpp"
#include <opencv2/highgui.hpp>
#include <opencv2/ml.hpp>
#include"util_define.h"
#include"util_for_cgal.h"
#include"gl_widget.h"
#include"save_svg.h"

template<typename Ini_data_cost>
struct Cost_vertices_empty_sample
{
	template<typename Triangulation, typename Vertex, typename DataLabel>
	static void p_label(Triangulation& t, Vertex& v, DataLabel& data_label)
	{
	}
	template<typename Label, typename Triangulation, typename Vertex>
	static Label recompute_label(Triangulation& t, Vertex& v)
	{
		return 0;
	}
	template<typename Triangulation, typename Vertex>
	static float cost(Triangulation& t, Vertex& v, decltype(Triangulation::vertex_data)& vertex_data_new = decltype(Triangulation::vertex_data)(), decltype(Triangulation::edge_data)& edge_data_new = decltype(Triangulation::edge_data)(), decltype(Triangulation::face_data)& face_data_new = decltype(Triangulation::face_data)())
	{
		return 0.0f;
	}
};

template<typename Ini_data_cost>
struct Cost_edges_empty_sample
{
	template<typename Triangulation, typename Edge, typename DataLabel>
	static void p_label(Triangulation& t, Edge& e, DataLabel& data_label)
	{
	}
	template<typename Label, typename Triangulation, typename Edge>
	static Label recompute_label(Triangulation& t, Edge& e)
	{
		return 0;
	}
	template<typename Triangulation, typename Edge>
	static float cost(Triangulation& t, Edge& e, decltype(Triangulation::vertex_data)& vertex_data_new = decltype(Triangulation::vertex_data)(), decltype(Triangulation::edge_data)& edge_data_new = decltype(Triangulation::edge_data)(), decltype(Triangulation::face_data)& face_data_new = decltype(Triangulation::face_data)())
	{
		return 0.0f;
	}
};
template<typename Ini_data_cost>
struct Cost_faces_empty_sample
{
	template<typename Triangulation, typename Face, typename DataLabel>
	static void p_label(Triangulation& t, Face& f, DataLabel& label)
	{
	}

	template<typename Label, typename Triangulation, typename Face>
	static Label recompute_label(Triangulation& t, Face& f)
	{
		return 0;
	}
	template<typename Triangulation, typename Face>
	static float cost(Triangulation& t, Face& f, decltype(Triangulation::vertex_data)& vertex_data_new = decltype(Triangulation::vertex_data)(), decltype(Triangulation::edge_data)& edge_data_new = decltype(Triangulation::edge_data)(), decltype(Triangulation::face_data)& face_data_new = decltype(Triangulation::face_data)())
	{
		return 0.0f;
	}
};

struct ini_data_cost_empty_sample
{
	//norm of the gradient of the image: used for sampling point
	static cv::Mat_<float> img_grad;
	//input image: used to obtain rows, cols and other stuff
	static cv::Mat_<cv::Vec3f> img_color;
	//main parameter to balance the fidelity term and the prior term defined in equation 8
	static double alpha;
	//parameter used to save files
	static std::string outut_base_name;
	//parameter used by the simulated annealing to compute the kernel ratio defined in equation 5
	static int target_nb_point;
	
	//this function is required, it is used by the system when the mark alteration kernel operator is called.
	template<typename SA>
	static void operator_label(SA* sa)
	{
		//no mark alteration kernel for the empty sample demo
	}	
	//this function is required.
	static void process_user_classes_input(std::set<glm::ivec2, less_ivec2 >& pixel_label4_1, std::set<glm::ivec2, less_ivec2>& pixel_label4_2)
	{
		
	}
	//this function is required, it is used by the system to check if the selected point is one of the four boundary point of the triangulation wich are at the exterior of the domain (pt is not necessarly composed by integers.
	template<typename Pt>
	static bool is_not_valid(const Pt& pt)
	{
		return pt.x() < 0 || pt.x() >= img_color.cols;
	}
	//this function is required, it is used by the system to check if a parturbation of a vertex do not put it at the exterior of the image domain
	static bool is_in_img(const int& x, const int& y)
	{
		return x >= 0 && y >= 0 && x < img_color.cols&&y < img_color.rows;
	}
	
	//this function is required, it is used by the system to initialize the DPP
	//the variables img_grad, img_color and outut_base_name should be initilized
	static void ini_from_file(const std::string base_name, const std::string& full_name)
	{
		img_color = cv::Mat_<cv::Vec3f>(1024, 1024, cv::Vec3f(0.0f, 0.0f, 0.0f));
		img_grad = cv::Mat_<float>(1024, 1024, 0.0f);
		outut_base_name = base_name;
	}

	//this function is required, it is used by the system to save some results
	template<typename SA>
	static void save_output(const SA& sa, const int rows, const int cols)
	{
		cv::imwrite((outut_base_name + "_input.png").c_str(), img_color * 255.0f);
		save_graph_no_label(outut_base_name, "_output.svg", sa.triangulation, rows, cols);
		sa.save_energy((outut_base_name + "_energy.csv").c_str());
	}

	//this function is required, it is called by the system at the begining. You can load specific parameters for your application
	static void load_app_specific_parameters()
	{
		std::string str, str2;
		std::istringstream iss;
		std::ifstream ifs("config/app_empty_sample_parameters.conf");
		while (std::getline(ifs, str2).good())
		{
			iss = std::istringstream();//otherwise iss  is not updated (or in a bad state after finisshing to read a line)
			iss.str(str2.c_str());
			std::getline(iss, str, ' ');
			if (str == "alpha")
				iss >> alpha;
		}
		ifs.close();
	}
};

#endif  __HEADER___COST_EMPTY_SAMPLE__

