#pragma once
#ifndef __HEADER___GUI_APPLI_IMAGE_COMPRESSION__
#define __HEADER___GUI_APPLI_IMAGE_COMPRESSION__

#include"gl_widget.h"

struct gui_rendering_image_compression
{
	static std::vector<std::pair<std::string, double*>> link_double_spinbox_in_gui;
	static std::vector<std::pair<std::string, int*>> link_int_spinbox_in_gui;

	static std::vector<std::string> extra_alpha_parameter;

	template<typename Triangulation>
	static std::vector<float*> ini_vbo_get_alpha(Triangulation& triangulation, gl_widget* gl_wgt, VBO& vbo_to_put_vertices, const std::string& obj_name, const float depth, const glm::vec3& color_nodes, const std::vector<glm::vec3>& color_edges)
	{
		const int img_cols = gl_wgt->img_cols;
		const int img_rows = gl_wgt->img_rows;
		std::vector<glm::vec2> vec_data_deg1;
		std::vector<glm::vec2> vec_data_triangle;
		std::vector<glm::vec3> color_triangle;
		std::vector<int> color_edge;

		vec_data_deg1.reserve((triangulation.number_of_faces() + 1 + triangulation.number_of_vertices() - 2) * 2);
		color_edge.reserve((triangulation.number_of_faces() + 1 + triangulation.number_of_vertices() - 2) * 2);

		for (auto it(triangulation.finite_edges_begin()), it_end(triangulation.finite_edges_end()); it != it_end; ++it)
		{
			const auto& face = *it->first;
			const int& idx = it->second;
			const auto& p1 = face.vertex((idx + 1) % 3)->point();
			const auto& p2 = face.vertex((idx + 2) % 3)->point();
			vec_data_deg1.emplace_back(p1.x() / img_cols, p1.y() / img_rows);
			vec_data_deg1.emplace_back(p2.x() / img_cols, p2.y() / img_rows);

			if (triangulation.vertex_data[get_vertex(*it, 1)->info()].label
				== triangulation.vertex_data[get_vertex(*it, 2)->info()].label)
				color_edge.emplace_back(0);
			else
				color_edge.emplace_back(1);
			color_edge.emplace_back(color_edge.back());
		}
		for (auto it(triangulation.finite_faces_begin()), it_end(triangulation.finite_faces_end()); it != it_end; ++it)
		{
			const typename Triangulation::Vertex_handle::value_type::Point& p1 = it->vertex(0)->point();
			const typename Triangulation::Vertex_handle::value_type::Point& p2 = it->vertex(1)->point();
			const typename Triangulation::Vertex_handle::value_type::Point& p3 = it->vertex(2)->point();
			vec_data_triangle.emplace_back(p1.x() / img_cols, p1.y() / img_rows);
			vec_data_triangle.emplace_back(p2.x() / img_cols, p2.y() / img_rows);
			vec_data_triangle.emplace_back(p3.x() / img_cols, p3.y() / img_rows);

			color_triangle.emplace_back(triangulation.vertex_data[it->vertex(0)->info()].label);
			color_triangle.emplace_back(triangulation.vertex_data[it->vertex(1)->info()].label);
			color_triangle.emplace_back(triangulation.vertex_data[it->vertex(2)->info()].label);

		}
		vbo_to_put_vertices.clear_vbo();
		vbo_to_put_vertices.add_vertex(obj_name + "_1", vec_data_deg1);
		vbo_to_put_vertices.add_vertex(obj_name + "_1_c", color_edge);
		vbo_to_put_vertices.add_vertex(obj_name + "_2", vec_data_triangle);
		vbo_to_put_vertices.add_vertex(obj_name + "_2_c", color_triangle);
		vbo_to_put_vertices.create_vbo();

		auto* tri = gl_wgt->ini_triangle_color(obj_name + "_2", vbo_to_put_vertices, depth, color_triangle.size());
		gl_bezier_deg_color_1* edges = gl_wgt->ini_bezier_deg_color<gl_bezier_deg_color_1>(obj_name + "_1", vbo_to_put_vertices, depth, color_edges, vec_data_deg1.size());
		edges->line_width = 1.0f;

		return std::vector<float*>{&edges->alpha, &tri->alpha};
	}

};
#endif  __HEADER___GUI_APPLI_IMAGE_COMPRESSION__

