#pragma once
#ifndef __HEADER___COST_APPLI_LINE_NETWORK_EXTRACTION__
#define __HEADER___COST_APPLI_LINE_NETWORK_EXTRACTION__

#include<CGAL/squared_distance_2.h>
#include<CGAL/Cartesian/line_constructions_2.h>

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>

#include"util_define.h"
#include"util_for_cgal.h"
#include"save_svg.h"


template<typename Ini_data_cost>
struct Cost_vertices_line_network_extraction
{
	template<typename Triangulation, typename Vertex, typename DataLabel>
	static void p_label(Triangulation& t, Vertex& v, DataLabel& data_label)
	{
	}
	template<typename Label, typename Triangulation, typename Vertex>
	static Label recompute_label(Triangulation& t, Vertex& v)
	{
		return 0;
	}
	template<typename Triangulation, typename Vertex>
	static float cost(Triangulation& t, Vertex& v, decltype(Triangulation::vertex_data)& vertex_data_new = decltype(Triangulation::vertex_data)(), decltype(Triangulation::edge_data)& edge_data_new = decltype(Triangulation::edge_data)(), decltype(Triangulation::face_data)& face_data_new = decltype(Triangulation::face_data)())
	{
		//in this function we compute alpha * w_c(p) of equation 10
		const auto& p = v.point();
		if (Ini_data_cost::is_not_valid(p))
			return 0.0f;
		std::list<Triangulation::Vertex*> lst_labelled_edges;
		elem_around(v.incident_vertices(), [&v, &t, &edge_data_new, &lst_labelled_edges](decltype(*v.incident_vertices())& v2)//ref to vertex
		{
			decltype(Triangulation::edge_data)::key_type me(v.info(), v2.info());
			decltype(Triangulation::edge_data)::const_iterator it = edge_data_new.find(me);
			if (it != edge_data_new.end())
			{
				if (it->second.label > 0)
				{
					lst_labelled_edges.emplace_back(&v2);
				}
			}
			else if (t.edge_data[me].label > 0)
			{
				lst_labelled_edges.emplace_back(&v2);
			}
		});
		float w_c = 0.0f;
		if (lst_labelled_edges.size() == 1)
			//w_c += 0.025f;// 0.05f;// 0.025f;// 0.0025f;
			w_c += 0.05f;// 0.05f;// 0.025f;// 0.0025f;
		if (lst_labelled_edges.size() > 1)
		{
			//	std::cout << "aa" << std::endl;
			const Vertex::Point& point_ref = v.point();
			double max_cos_angle_between_edges = -1.0;
			for (std::list<Triangulation::Vertex*>::const_iterator it(std::begin(lst_labelled_edges)), it_end(std::end(lst_labelled_edges)); it != it_end; ++it)
			{
				auto v1 = point_ref - (**it).point();
				double dst_f = sqrt(v1.squared_length());
				if (dst_f != 0)
					v1 = v1 / dst_f;
				//		std::cout << "cc" << std::endl;
				for (std::list<Triangulation::Vertex*>::const_iterator it_2(std::next(it)); it_2 != it_end; ++it_2)
				{
					decltype(v1) v2 = point_ref - (*it_2)->point();
					dst_f = sqrt(v2.squared_length());
					if (dst_f != 0)
						v2 = v2 / dst_f;
					//v2 /= dst_f;
					double tmp = v1 * v2;
					if (tmp > max_cos_angle_between_edges)
						max_cos_angle_between_edges = tmp;
				}
			}
			//const double eps = 0.965925826;// cos(M_PI / 12)
			//const double eps2 = 29.3477403; // 1 / ( 1 - cos(M_PI/12))
			//const double eps = cos(M_PI/5);// cos(M_PI / 12)
			//const double eps2 = 1.0 / (1 - cos(M_PI / 5)); // 1 / ( 1 - cos(M_PI/12))
			const double eps = cos(M_PI / 8);// cos(M_PI / 12)
			const double eps2 = 1.0 / (1 - cos(M_PI / 8)); // 1 / ( 1 - cos(M_PI/12))
			//todo replace linear per part function by sigmoid
			if (max_cos_angle_between_edges > eps)
				w_c += (float)((max_cos_angle_between_edges - eps) * eps2);
		}
		return w_c * Ini_data_cost::alpha;
	}
};
template<typename Ini_data_cost>
struct Cost_edges_line_network_extraction
{
	template<typename Edge>
	static double weight_edge_length(Edge& e)
	{
		const double le = sqrt((e.first->vertex((e.second + 2) % 3)->point() - e.first->vertex((e.second + 1) % 3)->point()).squared_length());
		if (le == 0)
			return 10.0;
		constexpr double param_exp = 4.59511985;//=std::log(99.0);
		constexpr double delta = 0.5;
		const double delta_lm = Ini_data_cost::min_length_edge*delta;
		const double delta_lm_2 = Ini_data_cost::min_length_edge*(1.0 - delta);
		return  1.0 / (1.0 + std::exp(-(delta_lm_2 - le) / delta_lm * param_exp));
	}
	template<typename Triangulation, typename Edge, typename DataLabel>
	static void p_label(Triangulation& t, Edge& e, DataLabel& data_label)
	{
		switch (data_label.label)
		{
		case 0:
			data_label.p_label = 1 - p_label_sub(t, e, 1);
		default:
			data_label.p_label = p_label_sub(t, e, 1);
		}
	}
	template<typename Triangulation, typename Edge, typename Label>
	static float p_label_sub(Triangulation& t, Edge& e, Label label)
	{

		const auto& ptmp1 = e.first->vertex((e.second + 1) % 3)->point();
		const auto& ptmp2 = e.first->vertex((e.second + 2) % 3)->point();

		const auto& p1 = ptmp1.x() < ptmp2.x() ? ptmp1 : (ptmp1.x() == ptmp2.x() && ptmp1.y() < ptmp2.y()) ? ptmp1 : ptmp2;
		const auto& p2 = p1 == ptmp1 ? ptmp2 : ptmp1;

		if (Ini_data_cost::is_not_valid(p1) || Ini_data_cost::is_not_valid(p2))
			return 0.0f;
		if (p2 == p1)
			return 0.0f;
		const auto& dir = (p2 - p1);
		const double dst_f = sqrt(dir.squared_length());
		const int dst = static_cast<int>(floor(dst_f));
		auto dir_norm = dir / dst_f;
		decltype(dir_norm) dir_norm_ortho(dir_norm.y(), -dir_norm.x());
		glm::vec2 dir_edge(dir_norm.x(), dir_norm.y());
		glm::vec2 normal_edge(dir_norm_ortho.x(), dir_norm_ortho.y());
		auto tmp_pt = p1;
		auto tmp_pt1 = p1;
		auto tmp_pt2 = p1;
		int nb_pt = 0;
		float proba_label_compl = 0.0f;
		const float theta_ref = (float)atan2(dir.y(), dir.x());
		for (int i = 0; i<dst; ++i)
		{
			const int x = static_cast<int>(floor(tmp_pt.x()));
			const int y = static_cast<int>(floor(tmp_pt.y()));
			tmp_pt1 = tmp_pt + dir_norm_ortho * label;
			tmp_pt2 = tmp_pt - dir_norm_ortho * label;

			const int x1 = static_cast<int>(floor(tmp_pt1.x()));
			const int y1 = static_cast<int>(floor(tmp_pt1.y()));
			const int x2 = static_cast<int>(floor(tmp_pt2.x()));
			const int y2 = static_cast<int>(floor(tmp_pt2.y()));
			if (Ini_data_cost::is_in_img(x1, y1))
			{
				cv::Vec2f grad_I = Ini_data_cost::img_grad_xy[y1][x1];
				const float norm = Ini_data_cost::img_grad[y1][x1];
				const float inner_prod_un_is_good = glm::dot(normal_edge, glm::vec2(grad_I[0], grad_I[1]));
				proba_label_compl += std::exp(-norm * inner_prod_un_is_good * inner_prod_un_is_good * 8);	
				++nb_pt;
			}
			if (Ini_data_cost::is_in_img(x2, y2))
			{
				cv::Vec2f grad_I = Ini_data_cost::img_grad_xy[y2][x2];
				const float norm = Ini_data_cost::img_grad[y2][x2];
				const float inner_prod_un_is_good = glm::dot(-normal_edge, glm::vec2(grad_I[0], grad_I[1]));
				proba_label_compl += std::exp(-norm * inner_prod_un_is_good * inner_prod_un_is_good * 8);
				++nb_pt;
			}
			tmp_pt = tmp_pt + dir_norm;
		}
		if (nb_pt != 0)
		{
			return 1.0f - (proba_label_compl *  1.0f / nb_pt);
		}
		return 0.0f;
	}
	template<typename Label, typename Triangulation, typename Edge>
	static Label recompute_label(Triangulation& t, Edge& e)
	{
		const float p_label_val = p_label_sub(t, e, 1);
		int tmp = rand() % 2;
		//if (p_label_val > 0.5)
		if (tmp > 0.5 && p_label_val !=0.0f)//and not an edge connected to an invalid vertex
			return 1;
		return 0;
	}
	template<typename Triangulation, typename Edge>
	static float cost(Triangulation& t, Edge& e, decltype(Triangulation::vertex_data)& vertex_data_new = decltype(Triangulation::vertex_data)(), decltype(Triangulation::edge_data)& edge_data_new = decltype(Triangulation::edge_data)(), decltype(Triangulation::face_data)& face_data_new = decltype(Triangulation::face_data)())
	{
		const auto& v1 = e.first->vertex((e.second + 1) % 3);
		const auto& v2 = e.first->vertex((e.second + 2) % 3);
		const auto& p1 = v1->point();
		const auto& p2 = v2->point();
		if (Ini_data_cost::is_not_valid(p1) && Ini_data_cost::is_not_valid(p2))
			return 0.0f;
		if (Ini_data_cost::is_not_valid(p1) || Ini_data_cost::is_not_valid(p2))
			return 0.0f;
		if (p2 == p1)
			return 10.0f;


		//w_l of equation 10
		float w_l = weight_edge_length(e);
		//u_fidelity of equation 9
		float u_fidelity = 0.0f;
		decltype(Triangulation::edge_data)::key_type me(v1->info(), v2->info());
		decltype(Triangulation::edge_data)::const_iterator it = edge_data_new.find(me);
		auto data_e = (it != edge_data_new.end()) ? it->second : t.edge_data[me];
		if (data_e.label > 0)
			u_fidelity += (Ini_data_cost::gamma - data_e.p_label);
		return w_l*0.75f*Ini_data_cost::alpha + u_fidelity *(1.0 - Ini_data_cost::alpha);
	}
};
template<typename Ini_data_cost>
struct Cost_faces_line_network_extraction
{
	template<typename Triangulation, typename Face, typename DataLabel>
	static void p_label(Triangulation& t, Face& f, DataLabel& label)
	{
	}

	template<typename Label, typename Triangulation, typename Face>
	static Label recompute_label(Triangulation& t, Face& f)
	{
		return Label();
	}
	template<typename Triangulation, typename Face>
	static float cost(Triangulation& t, Face& f, decltype(Triangulation::vertex_data)& vertex_data_new = decltype(Triangulation::vertex_data)(), decltype(Triangulation::edge_data)& edge_data_new = decltype(Triangulation::edge_data)(), decltype(Triangulation::face_data)& face_data_new = decltype(Triangulation::face_data)())
	{
		return 0.0f;
	}
};

struct ini_data_cost_line_network_extraction
{
	//norm of the gradient of the image: used for sampling point
	static cv::Mat_<float> img_grad;
	//input image: used to obtain rows, cols and other stuff
	static cv::Mat_<cv::Vec3f> img_color;
	//main parameter to balance the fidelity term and the prior term defined in equation 8
	static double alpha;
	//parameter used to save files
	static std::string outut_base_name;
	//parameter used by the simulated annealing to compute the kernel ratio defined in equation 5
	static int target_nb_point;
	
	
	//specific-application variables
	//gamma is the parameter defined in equation 9
//min length edge is the parameter b of the sigmoid S_{a,b} of the function w_l of the equation 10
	static cv::Mat_<cv::Vec2f> img_grad_xy;
	static double gamma;
	static int min_length_edge;


	//this function is required, it is used by the system when the mark alteration kernel operator is called.
	template<typename SA>
	static void operator_label(SA* sa)
	{
		sa->operator_label_edge();
	}
	//this function is required.
	static void process_user_classes_input(std::set<glm::ivec2, less_ivec2>& pixel_label4_1, std::set<glm::ivec2, less_ivec2>& pixel_label4_2)
	{
	}
	//this function is required, it is used by the system to check if the selected point is one of the four boundary point of the triangulation wich are at the exterior of the domain (pt is not necessarly composed by integers.
	template<typename Pt>
	static bool is_not_valid(const Pt& pt)
	{
		return pt.x() < 0 || pt.x() >= img_color.cols;
	}
	//this function is required, it is used by the system to check if a parturbation of a vertex do not put it at the exterior of the image domain
	static bool is_in_img(const int& x, const int& y)
	{
		return x >= 0 && y >= 0 && x < img_color.cols&&y < img_color.rows;
	}
	//this function is required, it is used by the system to initialize the DPP
	//the variables img_grad, img_color and outut_base_name should be initilized
	static void ini_from_file(const std::string base_name, const std::string& full_name)
	{
		cv::Mat ini_input = cv::imread(full_name);
		cv::Mat input;
		cv::Mat input_grad_x;
		cv::Mat input_grad_y;
		cv::Mat_<float> grad_x;
		cv::Mat_<float> grad_y;

		outut_base_name = base_name;

		ini_input.convertTo(img_color, CV_32FC3);
		img_color *= (1.0 / 255);
		cv::cvtColor(ini_input, input, CV_RGB2GRAY);
		cv::Sobel(input, input_grad_x, CV_16S, 1, 0, 3);
		cv::Sobel(input, input_grad_y, CV_16S, 0, 1, 3);
		img_grad = cv::Mat_<float>(input_grad_y.rows, input_grad_y.cols);
		img_grad_xy = cv::Mat_<cv::Vec2f>(input_grad_y.rows, input_grad_y.cols, cv::Vec2f(0.0f, 0.0f));
		input_grad_x.convertTo(grad_x, CV_32FC1);
		input_grad_y.convertTo(grad_y, CV_32FC1);
		input_grad_x *= (1.0 / 255);
		input_grad_y *= (1.0 / 255);
		float* ptr_grad_x = grad_x[0];
		float* ptr_grad_y = grad_y[0];
		float* ptr_grad_norm = img_grad[0];
		cv::Vec2f* ptr_grad_normxy = img_grad_xy[0];
		float max = std::numeric_limits<float>::min();
		for (int col = 0; col < grad_x.cols; ++col)
		{
			for (int row = 0; row < grad_x.rows; ++row)
			{
				const float norm = *ptr_grad_norm = sqrt(*ptr_grad_x**ptr_grad_x + *ptr_grad_y**ptr_grad_y);
				if (norm != 0)
					*ptr_grad_normxy = cv::Vec2f(*ptr_grad_x / norm, *ptr_grad_y / norm);
				if (norm > max)
					max = norm;
				++ptr_grad_x;
				++ptr_grad_y;
				++ptr_grad_norm;
				++ptr_grad_normxy;
			}
		}
		if (max != 0)
		{
			img_grad *= 1.0 / max;
		}
		cv::GaussianBlur(img_grad, img_grad, { 7, 7 }, 4, 4);
	}

	//this function is required, it is used by the system to save some results
	template<typename SA>
	static void save_output(const SA& sa, const int rows, const int cols)
	{
		cv::imwrite((outut_base_name + "_input.png").c_str(), img_color * 255.0f);
		save_graph_label_edge(outut_base_name, "_output.svg", sa.triangulation, rows, cols);
		save_graph_label_edge(outut_base_name, "_output_overlap.svg", sa.triangulation, rows, cols, true, true);
		save_graph_label_edge(outut_base_name, "_output_line.svg", sa.triangulation, rows, cols, false);
		sa.save_energy((outut_base_name + "_energy.csv").c_str());
	}

	//this function is required, it is called by the system at the begining. You can load specific parameters for your application
	static void load_app_specific_parameters()
	{
		std::string str, str2;
		std::istringstream iss;
		std::ifstream ifs("config/app_line_network_extraction_parameters.conf");
		while (std::getline(ifs, str2).good())
		{
			iss = std::istringstream();//otherwise iss  is not updated (or in a bad state after finisshing to read a line)
			iss.str(str2.c_str());
			std::getline(iss, str, ' ');
			if (str == "alpha")
				iss >> alpha;
			else if (str == "min_length_edge")
				iss >> min_length_edge;
			else if (str == "gamma")
				iss >> gamma;
		}
		ifs.close();
	}
};

#endif  __HEADER___COST_APPLI_LINE_NETWORK_EXTRACTION__

