#ifndef OBJECT_EXTEND_REPERE_ORIGIN_HEADER
#define OBJECT_EXTEND_REPERE_ORIGIN_HEADER

#include"repere_origin.h"
#include"Shader.h"
#include <typeindex>
#include <typeinfo>

struct object;

struct draw_null
{
	inline static void before(){}
	inline static void after(){}
};
struct draw_cw
{
	inline static void before(){ glFrontFace(GL_CW); }
	inline static void after(){ glFrontFace(GL_CCW); }
};
struct draw_ccw
{
	inline static void before(){ glFrontFace(GL_CCW); }
	inline static void after(){}
};
struct object
{
	object() :vao(nullptr), shader(nullptr){}
	object(VAO* ptr_vao) :vao(ptr_vao), shader(ptr_vao->get_shader()){}
	object(const object& o) = delete;
	object(const object&& o) = delete;
	object& operator=(const object& o) = delete;
	virtual ~object()
	{
		delete vao;
		shader = nullptr;
		vao = nullptr;
	}
	virtual void draw(const glm::mat4& scene) = 0;
	static std::map<std::type_index, Shader> map_type_shader;

	inline void set_vao(VAO* _vao_rval_ref)
	{
		vao = _vao_rval_ref;
	}
	inline VAO* get_vao()
	{
		return vao;
	}
	inline void set_shader(Shader* const& ptr_shader)
	{
		shader = ptr_shader;
	}
	void change_group(const char* group, const char* data_name)
	{
		vao->add_group(group, data_name);
		vao->re_link_vao();
	}
protected:
	VAO* vao;
	Shader* shader;
};
template<typename Child, typename Repere = repere_origin, typename Draw_Before_After = draw_null>
class base_object : public object, public Repere
{
public:
	base_object<Child, Repere, Draw_Before_After>() : object(){}
	base_object<Child, Repere, Draw_Before_After>(VAO* ptr_vao) : object(ptr_vao){}
	virtual ~base_object(){}
	virtual void draw(const glm::mat4& scene)
	{
		Draw_Before_After::before();
		shader->set_vao(vao);
		shader->enable();
		Repere::set_repere_for_draw(shader,scene);
		reinterpret_cast<Child*>(this)->draw_2(scene);
		Draw_Before_After::after();
		Shader::disable();
		VAO::disable();
	}
};


template<typename T>
Shader* get_default_shader()
{
	if (std::is_base_of<object, T>::value)
	{
		if (object::map_type_shader.find(std::type_index(typeid(T))) == object::map_type_shader.end())
			T::ini_shader();
		return &object::map_type_shader[std::type_index(typeid(T))];
	}
}
#endif

