#pragma once
#ifndef ____BERNSTEIN____H___
#define ____BERNSTEIN____H___

#include"binomial.h"
#include<iostream>
#include <Eigen/Dense>
#include <Eigen/Sparse>

//class Bernstein
//{
//public:
//	double t;
//	//binomial<N>::data;
//	//binomial_val<N,0>
//};

/**
assert than data has an operator [] which can take the value from 0 to N included
*/
//inline typename std::enable_if<std::conditional<std::is_same<N, 3>::value, std::false_type, std::true_type>::type, void>::type
template<int N = 0>
struct bernstein_polynome
{
	template<typename T>
	inline bernstein_polynome(double t, T& data)
	{
		double pow_t[N + 1];
		double pow_1mt[N + 1];
		const double& mt = 1 - t;
		pow_t[0] = 1;
		pow_t[1] = t;
		pow_1mt[N - 1] = mt;
		pow_1mt[N] = 1;
		for (int i = 2; i < N + 1; ++i)
		{
			pow_t[i] = pow_t[i - 1] * t;
			pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
		}
		const std::array<int, N + 1>& bin = binomial<N>::data;
		for (int i = 0; i < N + 1; ++i)
		{
			data[i] = bin[i] * pow_t[i] * pow_1mt[i];
		}
	}
};
template<>
struct bernstein_polynome<3>
{
	template<typename T>
	inline bernstein_polynome(double t, T& data)
	{
		const double t2 = t*t;
		const double t3 = t2*t;
		const double _3t = t * 3;
		const double _3t2 = t2 * 3;
		const double _3t3 = t3 * 3;
		data[0] = -t3 + _3t2 - _3t + 1;
		data[1] = _3t3 - 2 * _3t2 + _3t;
		data[2] = -_3t3 + _3t2;
		data[3] = t3;
	}
};
template<>
struct bernstein_polynome<2>
{
	template<typename T>
	inline bernstein_polynome(double t, T& data)
	{
		const double t2 = t*t;
		const double t12 = 2*t;
		data[0] = t2 - t12 + 1;
		data[1] = t12 - 2 * t2;
		data[2] = t2;
	}
};
template<>
struct bernstein_polynome<1>
{
	template<typename T>
	inline bernstein_polynome(double t, T& data)
	{
		data[0] = (1 - t);
		data[1] = t;
	}
};
template<>
struct bernstein_polynome<0>
{
	template<typename T>
	inline bernstein_polynome(double t, T& data, int N)
	{
		std::vector<double> pow_t(N + 1);
		std::vector<double> pow_1mt(N + 1);
		const double& mt = 1 - t;
		pow_t[0] = 1;
		pow_t[1] = t;
		pow_1mt[N - 1] = mt;
		pow_1mt[N] = 1;
		for (int i = 2; i < N + 1; ++i)
		{
			pow_t[i] = pow_t[i - 1] * t;
			pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
		}
		const int* bin;
		if (N<32)
			bin = binomial_array<32>::ptr_data[N];
		else
		{
			std::cerr << "bezier curve have to have a degree lower than 32" << std::endl;
		}
		for (int i = 0; i < N + 1; ++i)
		{
			data[i] = bin[i] * pow_t[i] * pow_1mt[i];
		}
	}
};
inline void bernstein_polynome_3_inv(float t, Eigen::Vector4f& data)
{
	//bernstein_polynome<3>(t, data);
	//	Eigen::Vector4f data2;

	const float t2 = t*t;
	const float t3 = t2*t;
	//EIGEN_ALIGN16 const float t_[4] = { t3, t2, t, 1.0f };
	//const __m128 tmp_t = _mm_load_ps(t_);
	const __m128 tmp_t = _mm_set_ps(1.0f, t, t2, t3);// t3, t2, t, 1.0f);
	const __m128 tmp_tab = _mm_move_ss(_mm_setzero_ps(), tmp_t);
	const __m128 _3t_ = _mm_mul_ps(tmp_t, _mm_set1_ps(3));
	const __m128 tmp__ = _mm_shuffle_ps(_3t_, _3t_, _MM_SHUFFLE(0, 1, 1, 2));
	const __m128 tmp_1_ = _mm_shuffle_ps(tmp_t, tmp_tab, _MM_SHUFFLE(1, 0, 0, 3));
	const __m128 tmp2__ = _mm_hsub_ps(tmp__, tmp_1_);
	const __m128 tmp_3_ = _mm_shuffle_ps(tmp2__, tmp2__, _MM_SHUFFLE(1, 0, 0, 2));
	const __m128 tmp4__ = _mm_hsub_ps(tmp_3_, tmp_3_);
	_mm_store_ps(&data(0), _mm_shuffle_ps(tmp2__, tmp4__, _MM_SHUFFLE(0, 1, 1, 3)));
	//for (int i = 0; i < 4; ++i)
	//{
	//	std::cout << data(i) << " " << data2(i) << " " << data(i) - data2(i) << std::endl;
	//}
}
inline void bernstein_polynome_3(float t, Eigen::Vector4f& data)
{
	//bernstein_polynome<3>(t, data);
//	Eigen::Vector4f data2;
	
	const float t2 = t*t;
	const float t3 = t2*t;
	//EIGEN_ALIGN16 const float t_[4] = { t3, t2, t, 1.0f };
	//const __m128 tmp_t = _mm_load_ps(t_);
	const __m128 tmp_t = _mm_set_ps(1.0f, t, t2, t3);// t3, t2, t, 1.0f);
	const __m128 tmp_tab = _mm_move_ss(_mm_setzero_ps(), tmp_t);
	const __m128 _3t_ = _mm_mul_ps(tmp_t, _mm_set1_ps(3));
	const __m128 tmp__ = _mm_shuffle_ps(_3t_, _3t_, _MM_SHUFFLE(0, 1, 1, 2));
	const __m128 tmp_1_ = _mm_shuffle_ps(tmp_t, tmp_tab, _MM_SHUFFLE(1, 0, 0, 3));
	const __m128 tmp2__ = _mm_hsub_ps(tmp__, tmp_1_);
	const __m128 tmp_3_ = _mm_shuffle_ps(tmp2__, tmp2__, _MM_SHUFFLE(1, 0, 0, 2));
	const __m128 tmp4__ = _mm_hsub_ps(tmp_3_, tmp_3_);
	_mm_store_ps(&data(0), _mm_shuffle_ps(tmp4__, tmp2__, _MM_SHUFFLE(3, 1, 1, 0)));
	//for (int i = 0; i < 4; ++i)
	//{
	//	std::cout << data(i) << " " << data2(i) << " " << data(i) - data2(i) << std::endl;
	//}
}
inline void bernstein_polynome_3(double t, Eigen::Vector4d& data)
{
	bernstein_polynome<3>(t, data);
}
//template<int N, typename T>
//void bernstein_polynome(double t, T& data)// double& data[N + 1])
//{
//	double pow_t[N + 1];
//	double pow_1mt[N + 1];
//	const double& mt = 1 - t;
//	pow_t[0] = 1;
//	pow_t[1] = t;
//	pow_1mt[N - 1] = mt;
//	pow_1mt[N] = 1;
//	for (int i = 2; i < N + 1; ++i)
//	{
//		pow_t[i] = pow_t[i - 1] * t;
//		pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
//	}
//	const std::array<int, N + 1>& bin = binomial<N>::data;
//	for (int i = 0; i < N + 1; ++i)
//	{
//		data[i] = bin[i] * pow_t[i] * pow_1mt[i];
//	}
//}
//template<typename T>
//inline void
//bernstein_polynome_3(double t, T& data)// double& data[N + 1])
//{
//	const double t2 = t*t;
//	const double t3 = t2*t;
//	const double _3t = t * 3;
//	const double _3t2 = t2 * 3;
//	const double _3t3 = t3 * 3;
//	data[0] = -t3 + _3t2 - _3t + 1;
//	data[1] = _3t3 - 2 * _3t2 + _3t;
//	data[2] = -_3t3 + _3t2;
//	data[3] = t3;
//}
//template<typename T>
//void bernstein_polynome(double t, T& data, int N)// double& data[N + 1])
//{
//	std::vector<double> pow_t(N + 1);
//	std::vector<double> pow_1mt(N + 1);
//	const double& mt = 1 - t;
//	pow_t[0] = 1;
//	pow_t[1] = t;
//	pow_1mt[N - 1] = mt;
//	pow_1mt[N] = 1;
//	for (int i = 2; i < N + 1; ++i)
//	{
//		pow_t[i] = pow_t[i - 1] * t;
//		pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
//	}
//	const int* bin;
//	if (N<32)
//		bin = binomial_array<32>::ptr_data[N];
//	else
//	{
//		std::cerr << "bezier curve have to have a degree lower than 32" << std::endl;
//	}
//	for (int i = 0; i < N + 1; ++i)
//	{
//		data[i] = bin[i] * pow_t[i] * pow_1mt[i];
//	}
//}
//template<int N>
//struct bernstein_polynome// double& data[N + 1])
//{
//	template<typename T>
//	void operator()(double t, T& data)
//	{
//		double pow_t[N + 1];
//		double pow_1mt[N + 1];
//		const double& mt = 1 - t;
//		pow_t[0] = 1;
//		pow_t[1] = t;
//		pow_1mt[N - 1] = mt;
//		pow_1mt[N] = 1;
//		for (int i = 2; i < N + 1; ++i)
//		{
//			pow_t[i] = pow_t[i - 1] * t;
//			pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
//		}
//		const std::array<int, N + 1>& bin = binomial<N>::data;
//		for (int i = 0; i < N + 1; ++i)
//		{
//			data[i] = bin[i] * pow_t[i] * pow_1mt[i];
//		}
//	}
//};
//template<typename T>
//struct bernstein_polynome_3
//{
//
//};
/**
assert than data hand data_grad have an operator [] which can take the value from 0 to N included
*/
template<int N, typename T>
void bernstein_polynome_and_grad(double t, T& data, T& data_grad)// double& data[N + 1])
{
	double pow_t[N + 1];
	double pow_1mt[N + 1];
	const double& mt = 1 - t;
	pow_t[0] = 1;
	pow_t[1] = t;
	pow_1mt[N - 1] = mt;
	pow_1mt[N] = 1;
	for (int i = 2; i < N + 1; ++i)
	{
		pow_t[i] = pow_t[i - 1] * t;
		pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
	}
	const std::array<int, N + 1>& bin = binomial<N>::data;
	for (int i = 0; i < N + 1; ++i)
	{                                                                                                                            
		data[i] = bin[i] * pow_t[i] * pow_1mt[i];
	}
	data_grad[0] = - N * bin[0] * pow_1mt[1];
	for (int i = 1; i < N; ++i)
	{
		data_grad[i] = bin[i] * (i * pow_t[i - 1] * pow_1mt[i] - (N - i) * pow_t[i] * pow_1mt[i + 1]);
	}
	data_grad[N] = N * bin[N] * pow_t[N - 1];
}

/**
assert than data_grad has an operator [] which can take the value from 0 to N included
*/
template<int N, typename T>
void bernstein_grad(double t, T& data_grad)// double& data[N + 1])
{
	double pow_t[N + 1];
	double pow_1mt[N + 1];
	const double& mt = 1 - t;
	pow_t[0] = 1;
	pow_t[1] = t;
	pow_1mt[N - 1] = mt;
	pow_1mt[N] = 1;
	for (int i = 2; i < N + 1; ++i)
	{
		pow_t[i] = pow_t[i - 1] * t;
		pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
	}
	const std::array<int, N + 1>& bin = binomial<N>::data;
	data_grad[0] = -N * bin[0] * pow_1mt[1];
	for (int i = 1; i < N; ++i)
	{
		data_grad[i] = bin[i] * (i * pow_t[i - 1] * pow_1mt[i] - (N - i) * pow_t[i] * pow_1mt[i + 1]);
	}
	data_grad[N] = N * bin[N] * pow_t[N - 1];
}
/**
assert than data_grad has an operator [] which can take the value from 0 to N included
*/
template<typename T>
void bernstein_grad(double t, T& data_grad, int N)// double& data[N + 1])
{
	std::vector<double> pow_t(N + 1);
	std::vector<double> pow_1mt(N + 1);
	const double& mt = 1 - t;
	pow_t[0] = 1;
	pow_t[1] = t;
	pow_1mt[N - 1] = mt;
	pow_1mt[N] = 1;
	for (int i = 2; i < N + 1; ++i)
	{
		pow_t[i] = pow_t[i - 1] * t;
		pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
	}
	//const std::array<int, N + 1>& bin = binomial<N>::data;
	const int* bin;
	//else if (N<8)
	//	bin = binomial_array<8>::ptr_data[N];
	//else if (N<16)
	//	bin = binomial_array<16>::ptr_data[N];
	if (N<32)
		bin = binomial_array<32>::ptr_data[N];
	else
	{
		std::cerr << "bezier curve have to have a degree lower than 32" << std::endl;
	}
	data_grad[0] = -N * bin[0] * pow_1mt[1];
	for (int i = 1; i < N; ++i)
	{
		data_grad[i] = bin[i] * (i * pow_t[i - 1] * pow_1mt[i] - (N - i) * pow_t[i] * pow_1mt[i + 1]);
	}
	data_grad[N] = N * bin[N] * pow_t[N - 1];
}
/**
assert than data has an operator [] which can take the value from 0 to N included
*/
/**
assert than data and data_grad have an operator [] which can take the value from 0 to N included
*/
template<typename T>
void bernstein_polynome_and_grad(double t, T& data, T& data_grad, int N)// double& data[N + 1])
{
	std::vector<double> pow_t(N + 1);
	std::vector<double> pow_1mt(N + 1);
	const double& mt = 1 - t;
	pow_t[0] = 1;
	pow_t[1] = t;
	pow_1mt[N - 1] = mt;
	pow_1mt[N] = 1;
	for (int i = 2; i < N + 1; ++i)
	{
		pow_t[i] = pow_t[i - 1] * t;
		pow_1mt[N - i] = pow_1mt[N - i + 1] * mt;
	}
	const int* bin;
	if (N<32)
		bin = binomial_array<32>::ptr_data[N];
	else
	{
		std::cerr << "bezier curve have to have a degree lower than 32" << std::endl;
	}
	for (int i = 0; i < N + 1; ++i)
	{
		data[i] = bin[i] * pow_t[i] * pow_1mt[i];
	}
	data_grad[0] = -N * bin[0] * pow_1mt[1];
	for (int i = 1; i < N; ++i)
	{
		data_grad[i] = bin[i] * (i * pow_t[i - 1] * pow_1mt[i] - (N - i) * pow_t[i] * pow_1mt[i + 1]);
	}
	data_grad[N] = N * bin[N] * pow_t[N - 1];
}
#endif