#pragma once
#ifndef ___OBJ_BEZIER___HEADER___
#define ___OBJ_BEZIER___HEADER___
#include "object.h"


class gl_triangle_color : public base_object<gl_triangle_color, repere_2d>
{
public:
	gl_triangle_color(VAO* ptr_vao, const GLsizei& _nb_line = 0, float _depth = 0.0f, const float _alpha = 1.0f, const float& _line_width = 5)
	: base_object<gl_triangle_color, repere_2d>(ptr_vao), nb_line(_nb_line), depth(_depth), line_width(_line_width), alpha(_alpha)
{}
	   virtual ~gl_triangle_color() {}
	   GLsizei nb_line;
	   float line_width;
	   float alpha;
	   float depth;
	   static void ini_shader()
	   {
		   Shader s;
		 //  s.loadVertexShaderFromFile("shader/shader_triangle_color_gray.vert");
		 //  s.loadFragmentShaderFromFile("shader/shader_triangle_color_gray.frag");
		   s.loadVertexShaderFromFile("shader/shader_triangle_color.vert");
		   s.loadFragmentShaderFromFile("shader/shader_triangle_color.frag");
		   object::map_type_shader.emplace(std::make_pair(std::type_index(typeid(gl_triangle_color)), std::move(s)));
	   }
	   inline void draw_2(const glm::mat4& scene)const
	   {
		   shader->set_uniform("alpha", alpha);
		   //shader->set_uniform("depth", depth);
		   glLineWidth(line_width);
		   //glDrawArrays(GL_POINTS, 0, nb_l/2ine);
		   glDrawArrays(GL_TRIANGLES, 0, nb_line);
		   glLineWidth(1.0);
	   }
};

class gl_bezier : public base_object<gl_bezier, repere_2d>
{
public:
	//obj_bezier();
	gl_bezier(VAO* ptr_vao, const GLsizei& _nb_line = 0, float _depth = 0.0f, const glm::vec3& color = { 0, 0, 1.0f }, const float _alpha = 1.0f, const float& _line_width = 5)
		: base_object<gl_bezier, repere_2d>(ptr_vao), nb_line(_nb_line), depth(_depth), line_width(_line_width), color_curve(color), alpha(_alpha)
	{}
	virtual ~gl_bezier(){}
	GLsizei nb_line;
	float line_width;
	glm::vec3 color_curve;
	float alpha;
	float depth;
	static void ini_shader()
	{
		Shader s;
		s.loadVertexShaderFromFile("shader/shader_bezier_from_ctpt.vert");
		s.loadGeometryShaderFromFile("shader/shader_bezier_from_ctpt.geom");
		s.loadFragmentShaderFromFile("shader/shader_bezier_from_ctpt.frag");
		object::map_type_shader.emplace(std::make_pair(std::type_index(typeid(gl_bezier)), std::move(s)));
	}
	inline void draw_2(const glm::mat4& scene)const
	{
		shader->set_uniform("color_curve", color_curve);
		//std::cout << alpha << std::endl;
		shader->set_uniform("alpha", alpha);
		shader->set_uniform("depth", depth);
		glLineWidth(line_width);
		//glDrawArrays(GL_POINTS, 0, nb_l/2ine);
		glDrawArrays(GL_LINES, 0, nb_line*2);
		glLineWidth(1.0);
	}
};


class gl_bezier_deg_3 : public base_object<gl_bezier_deg_3, repere_2d>
{
public:
	//obj_bezier();
	gl_bezier_deg_3(VAO* ptr_vao, const GLsizei& _nb_line = 0, float _depth = 0.0f, const glm::vec3& color = { 0, 0, 1.0f }, const float _alpha = 1.0f, const float& _line_width = 5)
		: base_object<gl_bezier_deg_3, repere_2d>(ptr_vao), nb_line(_nb_line), depth(_depth), line_width(_line_width), color_curve(color), alpha(_alpha)
	{}
	virtual ~gl_bezier_deg_3(){}
	GLsizei nb_line;
	float line_width;
	glm::vec3 color_curve;
	float alpha;
	float depth;
	static void ini_shader()
	{
		Shader s;
		s.loadVertexShaderFromFile("shader/shader_bezier_from_ctpt_deg3.vert");
		s.loadGeometryShaderFromFile("shader/shader_bezier_from_ctpt_deg3.geom");
		s.loadFragmentShaderFromFile("shader/shader_bezier_from_ctpt_deg3.frag");
		object::map_type_shader.emplace(std::make_pair(std::type_index(typeid(gl_bezier_deg_3)), std::move(s)));
	}
	inline void draw_2(const glm::mat4& scene)const
	{
		shader->set_uniform("color_curve", color_curve);
		//std::cout << alpha << std::endl;
		shader->set_uniform("alpha", alpha);
		shader->set_uniform("depth", depth);
		glLineWidth(line_width);
		//glDrawArrays(GL_POINTS, 0, nb_l/2ine);
		glDrawArrays(GL_LINES, 0, nb_line);
		glLineWidth(1.0);
	}
};


class gl_bezier_deg_2 : public base_object<gl_bezier_deg_2, repere_2d>
{
public:
	//obj_bezier();
	gl_bezier_deg_2(VAO* ptr_vao, const GLsizei& _nb_line = 0, float _depth = 0.0f, const glm::vec3& color = { 0, 0, 1.0f }, const float _alpha = 1.0f, const float& _line_width = 5)
		: base_object<gl_bezier_deg_2, repere_2d>(ptr_vao), nb_line(_nb_line), depth(_depth), line_width(_line_width), color_curve(color), alpha(_alpha)
	{}
	virtual ~gl_bezier_deg_2(){}
	GLsizei nb_line;
	float line_width;
	glm::vec3 color_curve;
	float alpha;
	float depth;
	static void ini_shader()
	{
		Shader s;
		s.loadVertexShaderFromFile("shader/shader_bezier_from_ctpt_deg2.vert");
		s.loadGeometryShaderFromFile("shader/shader_bezier_from_ctpt_deg2.geom");
		s.loadFragmentShaderFromFile("shader/shader_bezier_from_ctpt_deg2.frag");
		object::map_type_shader.emplace(std::make_pair(std::type_index(typeid(gl_bezier_deg_2)), std::move(s)));
	}
	inline void draw_2(const glm::mat4& scene)const
	{
		shader->set_uniform("color_curve", color_curve);
		//std::cout << alpha << std::endl;
		shader->set_uniform("alpha", alpha);
		shader->set_uniform("depth", depth);
		glLineWidth(line_width);
		glDrawArrays(GL_TRIANGLES, 0, nb_line);
		glLineWidth(1.0);
	}
};

class gl_bezier_deg_color_1 : public base_object<gl_bezier_deg_color_1, repere_2d>
{
public:
	//obj_bezier();
	gl_bezier_deg_color_1(VAO* ptr_vao, const GLsizei& _nb_line = 0, float _depth = 0.0f, const std::vector<glm::vec3> color = std::vector<glm::vec3>(1,{ 0, 0, 1.0f }), const float _alpha = 1.0f, const float& _line_width = 5)
		: base_object<gl_bezier_deg_color_1, repere_2d>(ptr_vao), nb_line(_nb_line), depth(_depth), line_width(_line_width), color_curve(color), alpha(_alpha)
	{}
	virtual ~gl_bezier_deg_color_1() {}
	GLsizei nb_line;
	float line_width;
	std::vector<glm::vec3> color_curve;
	float alpha;
	float depth;
	static void ini_shader()
	{
		Shader s;
		s.loadVertexShaderFromFile("shader/shader_bezier_from_ctpt_deg1_color.vert");
		s.loadFragmentShaderFromFile("shader/shader_bezier_from_ctpt_deg1_color.frag");
		object::map_type_shader.emplace(std::make_pair(std::type_index(typeid(gl_bezier_deg_color_1)), std::move(s)));
	}
	inline void draw_2(const glm::mat4& scene)const
	{
		shader->set_uniform("color_curve", color_curve);
		shader->set_uniform("alpha", alpha);
		//shader->set_uniform("depth", depth);
		glLineWidth(line_width);
		//glDrawArrays(GL_POINTS, 0, nb_l/2ine);
		glDrawArrays(GL_LINES, 0, nb_line);// *2);
		glLineWidth(1.0);
	}
};

class gl_bezier_deg_1 : public base_object<gl_bezier_deg_1, repere_2d>
{
public:
	//obj_bezier();
	gl_bezier_deg_1(VAO* ptr_vao, const GLsizei& _nb_line = 0, float _depth = 0.0f, const glm::vec3& color = { 0, 0, 1.0f }, const float _alpha = 1.0f, const float& _line_width = 5)
		: base_object<gl_bezier_deg_1, repere_2d>(ptr_vao), nb_line(_nb_line), depth(_depth), line_width(_line_width), color_curve(color), alpha(_alpha)
	{}
	virtual ~gl_bezier_deg_1(){}
	GLsizei nb_line;
	float line_width;
	glm::vec3 color_curve;
	float alpha;
	float depth;
	static void ini_shader()
	{
		Shader s;
		s.loadVertexShaderFromFile("shader/shader_bezier_from_ctpt_deg1.vert");
		s.loadFragmentShaderFromFile("shader/shader_bezier_from_ctpt_deg1.frag");
		object::map_type_shader.emplace(std::make_pair(std::type_index(typeid(gl_bezier_deg_1)), std::move(s)));
	}
	inline void draw_2(const glm::mat4& scene)const
	{
		shader->set_uniform("color_curve", color_curve);
		shader->set_uniform("alpha", alpha);
		//shader->set_uniform("depth", depth);
		glLineWidth(line_width);
		//glDrawArrays(GL_POINTS, 0, nb_l/2ine);
		glDrawArrays(GL_LINES, 0, nb_line);
		glLineWidth(1.0);
	}
};
class gl_bezier_deg_0 : public base_object<gl_bezier_deg_0, repere_2d>
{
public:
	//obj_bezier();
	gl_bezier_deg_0(VAO* ptr_vao, const GLsizei& _nb_points = 0, float _depth = 0.0f, const glm::vec3& color = { 0.5f, 0.5f, 0.5f }, const float _alpha = 1.0f, const float& _radius_of_point = 0.01f)
		: base_object<gl_bezier_deg_0, repere_2d>(ptr_vao), nb_line(_nb_points), depth(_depth), radius_of_point(_radius_of_point), color_curve(color), alpha(_alpha), aspect_ratio(nullptr)
	{}
	virtual ~gl_bezier_deg_0() {}
	GLsizei nb_line;//nb points
	float radius_of_point;
	glm::vec3 color_curve;
	float alpha;
	float depth;
	float* aspect_ratio;
	static void ini_shader()
	{
		Shader s;
		s.loadVertexShaderFromFile("shader/shader_point.vert");
		s.loadGeometryShaderFromFile("shader/shader_point.geom");
		s.loadFragmentShaderFromFile("shader/shader_point.frag");
		object::map_type_shader.emplace(std::make_pair(std::type_index(typeid(gl_bezier_deg_0)), std::move(s)));
	}
	inline void draw_2(const glm::mat4& scene)const
	{
		shader->set_uniform("color_curve", color_curve);
		shader->set_uniform("alpha", alpha);
		//shader->set_uniform("depth", depth);

		shader->set_uniform("radius_of_point", radius_of_point);
		shader->set_uniform("ratio", *aspect_ratio);

		glDrawArrays(GL_POINTS, 0, nb_line);
	}
};


#endif