#pragma once
#ifndef _HEADER_SAVE_SVG_FILE___
#define _HEADER_SAVE_SVG_FILE___

#include"util_define.h"
#include"gl_widget.h"
#include"get_color.h"
inline void draw_svg_circle_(std::ofstream& ofs, int id, const std::string& color, const glm::vec2& c, const float& r, const float& stroke_width, const double opacity = 1)
{
	ofs << "<circle ";
	if (id != -1)
		ofs << "id = \"circle_" << id << "\" ";
	ofs << "cx=\"" << c.x << "\" cy=\"" << c.y << "\" r=\"" << r << "\"";
	ofs << " stroke=\"" << color << "\" fill=\"" << color << "\" stroke-width=\"" << stroke_width << "\" opacity=\"" << opacity << "\"/>" << std::endl;
}
inline void draw_svg_line_(std::ofstream& ofs, int id, const std::string& color, const glm::vec2& p1, const glm::vec2& p2, const int wigth = 2, const double opacity = 1)
{
	ofs << "<path ";
	if (id != -1)
		ofs << "id = \"curve_" << id << "\" ";
	ofs << "d = \"M";
	ofs << p1.x << "," << p1.y << " L";
	ofs << p2.x << "," << p2.y << "\"";
	ofs << " stroke=\"" << color << "\" stroke-width=\"" << wigth << "\" fill=\"none\" stroke-opacity=\"" << opacity << "\"/>" << std::endl;
}

template<typename Point>
inline void draw_svg_circle_(std::ofstream& ofs, int id, const std::string& color, const Point& c, const float& r, const float& stroke_width, const double opacity = 1)
{
	ofs << "<circle ";
	if (id != -1)
		ofs << "id = \"circle_" << id << "\" ";
	ofs << "cx=\"" << c.x() << "\" cy=\"" << c.y() << "\" r=\"" << r << "\"";
	ofs << " stroke=\"" << color << "\" fill=\"" << color << "\" stroke-width=\"" << stroke_width << "\" opacity=\"" << opacity << "\"/>" << std::endl;
}
template<typename Point>
inline void draw_svg_line_(std::ofstream& ofs, int id, const std::string& color, const Point& p1, const Point& p2, const int wigth = 2, const double opacity = 1)
{
	ofs << "<path ";
	if (id != -1)
		ofs << "id = \"curve_" << id << "\" ";
	ofs << "d = \"M";
	ofs << p1.x() << "," << p1.y() << " L";
	ofs << p2.x() << "," << p2.y() << "\"";
	ofs << " stroke=\"" << color << "\" stroke-width=\"" << wigth << "\" fill=\"none\" 	stroke-opacity=\""<< opacity  << "\"/>" << std::endl;
}
inline void draw_svg_bezier2(std::ofstream& ofs, int id, const std::string& color, const glm::vec2& p1, const glm::vec2& p2, const glm::vec2& p3)
{
	ofs << "<path ";
	if (id != -1)
		ofs << "id = \"curve_" << id << "\" ";
	ofs << "d = \"M";
	ofs << p1.x << "," << p1.y << " Q";
	ofs << p2.x << "," << p2.y << ' ';
	ofs << p3.x << "," << p3.y << "\"";
	ofs << " stroke=\"" << color << "\" stroke-width=\"2\" fill=\"none\"/>" << std::endl;
}
inline void draw_svg_bezier3(std::ofstream& ofs, int id, const std::string& color, const glm::vec2& p1, const glm::vec2& p2, const glm::vec2& p3, const glm::vec2& p4)
{
	ofs << "<path ";
	if (id != -1)
		ofs << "id = \"curve_" << id << "\" ";
	ofs << "d = \"M";
	ofs << p1.x << "," << p1.y << " C";
	ofs << p2.x << "," << p2.y << ' ';
	ofs << p3.x << "," << p3.y << ' ';
	ofs << p4.x << "," << p4.y << "\"";
	ofs << " stroke=\"" << color << "\" stroke-width=\"2\" fill=\"none\"/>" << std::endl;
}
inline void draw_svg_line_(std::ofstream& ofs, int id, const std::string& color, const double& p1x, const double& p1y, const double& p2x, const double& p2y, const int wigth = 2, const double opacity = 1)
{
	ofs << "<path ";
	if (id != -1)
		ofs << "id = \"curve_" << id << "\" ";
	ofs << "d = \"M";
	ofs << p1x << "," << p1y << " L";
	ofs << p2x << "," << p2y << "\"";
	ofs << " stroke=\"" << color << "\" stroke-width=\"" << wigth << "\" fill=\"none\" stroke-opacity=\"" << opacity << "\"/>" << std::endl;
}
inline void draw_svg_circle_(std::ofstream& ofs, int id, const std::string& color, const double& cx, const double& cy, const float& r, const float& stroke_width, const double opacity = 1)
{
	ofs << "<circle ";
	if (id != -1)
		ofs << "id = \"circle_" << id << "\" ";
	ofs << "cx=\"" << cx << "\" cy=\"" << cy << "\" r=\"" << r << "\"";
	ofs << " stroke=\"" << color << "\" fill=\"" << color << "\" stroke-width=\"" << stroke_width << "\" opacity=\"" << opacity << "\"/>" << std::endl;
}
template<typename Triangulation>
inline void save_curves(std::string base_name, std::string suffix, const Triangulation& triangulation, const int rows, const int cols, bool node = true, bool inactive_edge = true, bool color_vertex = false, bool overlap = false)
{
	int id = 0;
	std::ofstream ofs(base_name + suffix);
	ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
	
	for (auto it(triangulation.list_of_curve.begin()), it_end(triangulation.list_of_curve.end()); it != it_end; ++it)
	{
		const int type = it->second.type;
		if (type == 0)
			continue;
		else if (type == 1)
			draw_svg_line_(ofs, id++, "blue", it->second.p1, it->second.p2);
		else if (type == 2)
			draw_svg_bezier2(ofs, id++, "blue", it->second.p1, it->second.ctp1, it->second.p2);
		else if (type == 3)
			draw_svg_bezier3(ofs, id++, "blue", it->second.p1, it->second.ctp1, it->second.ctp2, it->second.p2);
	}
	ofs << "</svg>" << std::endl;
	ofs.close();
}

template<typename Triangulation>
inline void save_graph_no_label_vertex_color(std::string base_name, std::string suffix, const Triangulation& triangulation, const int rows, const int cols, bool overlap = false)
{
	int id = 0;
	std::ofstream ofs(base_name + suffix);
	ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
	if (overlap)
		ofs << "<image xlink:href=\"" << base_name << "_input.png" << "\" x=\"0\" y=\"0\" height=\"" << rows << "px\" width=\"" << cols << "px\"/>\n";
	for (auto it(triangulation.edges_begin()), it_end(triangulation.edges_end()); it != it_end; ++it)
	{
		typename Triangulation::Edge& e = *it;
		const typename Triangulation::Point& ptmp1 = e.first->vertex((e.second + 1) % 3)->point();
		const typename Triangulation::Point& ptmp2 = e.first->vertex((e.second + 2) % 3)->point();
		draw_svg_line_(ofs, id++, "darkgray", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 1, 0.5);
	}
	for (auto it(triangulation.vertices_begin()), it_end(triangulation.vertices_end()); it != it_end; ++it)
	{
		draw_svg_circle_(ofs, id++, get_color_html_code_from_bgr(triangulation.vertex_data.find(it->info())->second.label), it->point(), 1, 0, 1);
	//	draw_svg_circle_(ofs, id++, "darkgray", it->point(), 1, 0, 0.5);
	}
	ofs << "</svg>" << std::endl;
	ofs.close();
}

template<typename Triangulation>
inline void save_graph_no_label(std::string base_name, std::string suffix, const Triangulation& triangulation, const int rows, const int cols, bool overlap = false)
{
	int id = 0;
	std::ofstream ofs(base_name + suffix);
	ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
	if (overlap)
		ofs << "<image xlink:href=\"" << base_name << "_input.png" << "\" x=\"0\" y=\"0\" height=\"" << rows << "px\" width=\"" << cols << "px\"/>\n";
	for (auto it(triangulation.edges_begin()), it_end(triangulation.edges_end()); it != it_end; ++it)
	{
		typename Triangulation::Edge& e = *it;
		const typename Triangulation::Point& ptmp1 = e.first->vertex((e.second + 1) % 3)->point();
		const typename Triangulation::Point& ptmp2 = e.first->vertex((e.second + 2) % 3)->point();
		draw_svg_line_(ofs, id++, "darkgray", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 1, 0.5);
	}
	for (auto it(triangulation.vertices_begin()), it_end(triangulation.vertices_end()); it != it_end; ++it)
	{
		draw_svg_circle_(ofs, id++, "darkgray", it->point(), 1, 0, 0.5);
	}
	ofs << "</svg>" << std::endl;
	ofs.close();
}
template<typename Triangulation>
inline void save_graph_label_facet(std::string base_name, std::string suffix, const Triangulation& triangulation, const int rows, const int cols, bool inactive_edge = true, bool overlap = false)
{
	int id = 0;
	std::ofstream ofs(base_name + suffix);
	ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
	if (overlap)
		ofs << "<image xlink:href=\"" << base_name << "_input.png" << "\" x=\"0\" y=\"0\" height=\"" << rows << "px\" width=\"" << cols << "px\"/>\n";
	for (auto it(triangulation.edges_begin()), it_end(triangulation.edges_end()); it != it_end; ++it)
	{
		typename Triangulation::Edge& e = *it;
		const typename Triangulation::Point& ptmp1 = e.first->vertex((e.second + 1) % 3)->point();
		const typename Triangulation::Point& ptmp2 = e.first->vertex((e.second + 2) % 3)->point();
		//bool b = ptmp1.x() < 0 || ptmp1.x() > cols || ptmp2.x() < 0 || ptmp2.x() > cols;
		bool b = triangulation.is_infinite(it->first) || triangulation.is_infinite(it->first->neighbor(it->second));
		if (b)std::cout << "infinite" << std::endl;
		if ((!b) && triangulation.face_data.find(it->first->info())->second.label.label
			!= triangulation.face_data.find(it->first->neighbor(it->second)->info())->second.label.label)
			draw_svg_line_(ofs, id++, "red", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 2, 1.0);
		else if (inactive_edge)
			draw_svg_line_(ofs, id++, "darkgray", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 1, 0.5);
	}
	for (auto it(triangulation.vertices_begin()), it_end(triangulation.vertices_end()); it != it_end; ++it)
	{
		bool active_node = false;
		const typename Triangulation::Point point = it->point();
		//	const int x = (int)point.x();
		if (!(point.x() < 0 || point.x() >= cols))
		{
			typename Triangulation::Edge_circulator c = it->incident_edges();
			typename Triangulation::Edge_circulator c_end = c;
			do
			{
				if (triangulation.face_data.find(c->first->info())->second.label.label
					!= triangulation.face_data.find(c->first->neighbor(c->second)->info())->second.label.label)
				{
					active_node = true;
					break;
				}
			} while (++c != c_end);
		}
		if (active_node)
			draw_svg_circle_(ofs, id++, "red", it->point(), 2, 0);
		else if (inactive_edge)
			draw_svg_circle_(ofs, id++, "darkgray", it->point(), 1, 0, 0.5);
	}
	ofs << "</svg>" << std::endl;
	ofs.close();
}
template<typename Triangulation>
inline void save_graph_label_edge(std::string base_name, std::string suffix, const Triangulation& triangulation, const int rows, const int cols, bool inactive_edge = true, bool overlap = false)
{
	int id = 0;
	std::ofstream ofs(base_name + suffix);
	ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
	if (overlap)
		ofs << "<image xlink:href=\"" << base_name << "_input.png" << "\" x=\"0\" y=\"0\" height=\"" << rows << "px\" width=\"" << cols << "px\"/>\n";


	for (auto it(triangulation.edges_begin()), it_end(triangulation.edges_end()); it != it_end; ++it)
	{
		const typename Triangulation::Point& ptmp1 = it->first->vertex((it->second + 1) % 3)->point();
		const typename Triangulation::Point& ptmp2 = it->first->vertex((it->second + 2) % 3)->point();
		bool b = ptmp1.x() < 0 || ptmp1.x() > cols || ptmp2.x() < 0 || ptmp2.x() > cols;
		typename Triangulation::Edge& e = *it;
		auto data_e = triangulation.edge_data.find(Mon_edge(it->first->vertex((it->second + 1) % 3)->info(), it->first->vertex((it->second + 2) % 3)->info()))->second;
		//const int i = triangulation.label_on_edge(e);
		if (b)
		{
			if (inactive_edge)
				draw_svg_line_(ofs, id++, "darkgray", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 1, 0.5);
		}
		else
		{
			switch (data_e.label)
			{
			case 0:
				if (inactive_edge)
					draw_svg_line_(ofs, id++, "darkgray", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 1, 0.5);
				break;
			case 1:
				draw_svg_line_(ofs, id++, "red", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 2);
				break;
			case 2:
				draw_svg_line_(ofs, id++, "green", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 4);
				break;
			case 3:
				draw_svg_line_(ofs, id++, "orange", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 6);
				break;
			case 4:
				draw_svg_line_(ofs, id++, "cyan", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 8);
				break;

			}
		}
	}
	for (auto it(triangulation.vertices_begin()), it_end(triangulation.vertices_end()); it != it_end; ++it)
	{
		bool active_node = false;
		const typename Triangulation::Point point = it->point();
		if (!(point.x() < 0 || point.x() >= cols))
		{
			typename Triangulation::Edge_circulator c = it->incident_edges();
			typename Triangulation::Edge_circulator c_end = c;
			do
			{
				if (triangulation.edge_data.find(Mon_edge(c->first->vertex((c->second + 1) % 3)->info(), c->first->vertex((c->second + 2) % 3)->info()))->second.label != 0)
				{
					active_node = true;
					break;
				}
			} while (++c != c_end);
		}
		if (active_node)
			draw_svg_circle_(ofs, id++, "red", it->point(), 2, 0);
		else if (inactive_edge)
			draw_svg_circle_(ofs, id++, "darkgray", it->point(), 1, 0, 0.5);
	}
	ofs << "</svg>" << std::endl;
	ofs.close();
}
template<typename Triangulation>
inline void save_graph_only_vertex_color(const char* file_name, Triangulation& triangulation, const int rows, bool binary)
{
	std::ofstream ofs;
	if (binary)
		ofs.open(file_name,std::ios::binary);
	else
		ofs.open(file_name);
	for (auto it(triangulation.vertices_begin()), it_end(triangulation.vertices_end()); it != it_end; ++it)
	{
		const typename Triangulation::Point point = it->point();
		const glm::vec3 color = triangulation.vertex_data.find(it->info())->second.label;
		const glm::ivec3 colori = glm::max(glm::min(glm::ivec3(255, 255, 255), glm::ivec3(color.b*255.0f, color.g*255.0f, color.r*255.0f)), glm::ivec3(0, 0, 0));
		if (binary)
		{
			float x = point.x();
			float y = rows - point.y();
			unsigned char r = colori.r;
			unsigned char g = colori.g;
			unsigned char b = colori.b;
			ofs.write((const char*)&x, sizeof(float));
			ofs.write((const char*)&y, sizeof(float));
			ofs.write((const char*)&r, sizeof(unsigned char));
			ofs.write((const char*)&g, sizeof(unsigned char));
			ofs.write((const char*)&b, sizeof(unsigned char));
		}
		else
			ofs << point.x() << " " << rows - point.y() << " " << colori.r << " " << colori.g << " " << colori.b << std::endl;
	}
	ofs.close();
}
template<typename Triangulation>
inline void save_graph_ply(const char* file_name, const Triangulation& triangulation)
{
	std::ofstream ofs(file_name);
	ofs << "ply\nformat ascii 1.0\nelement vertex " << triangulation.number_of_vertices() << std::endl;
	ofs << "property float x\nproperty float y\nproperty float z" << std::endl;
	ofs << "property uchar red\nproperty uchar green\nproperty uchar blue" << std::endl;
	ofs << "element face " << triangulation.number_of_faces() << std::endl;
	ofs << "property list uchar int vertex_index\nend_header" << std::endl;
	std::map<int, int> id_vertex2ply;
	for (auto it(triangulation.vertices_begin()), it_end(triangulation.vertices_end()); it != it_end; ++it)
	{
		const typename Triangulation::Point point = it->point();
		const glm::vec3 color = triangulation.vertex_data.find(it->info())->second.label;
		const glm::ivec3 colori = glm::max(glm::min(glm::ivec3(255,255,255),glm::ivec3(color.b*255.0f, color.g*255.0f, color.r*255.0f)),glm::ivec3(0,0,0));
	
		ofs << point.x() << " " << -point.y() << " 0 " << colori.r << " " << colori.g << " " << colori.b << std::endl;
		id_vertex2ply.emplace(it->info(), (int)id_vertex2ply.size());
	}

	for (auto it(triangulation.faces_begin()), it_end(triangulation.faces_end()); it != it_end; ++it)
	{
		const typename Triangulation::Vertex_handle v1 = it->vertex(0);
		const typename Triangulation::Vertex_handle v2 = it->vertex(1);
		const typename Triangulation::Vertex_handle v3 = it->vertex(2);
		ofs << "3 " << id_vertex2ply[v1->info()] << " " << id_vertex2ply[v2->info()] << " " << id_vertex2ply[v3->info()] << std::endl;
	}
	ofs.close();
}
//template<typename Triangulation>
//inline void save_graph_line(const char* file_name, Triangulation& triangulation, const int rows, const int cols, bool node = true)
//{
//	int id = 0;
//	std::ofstream ofs(file_name);
//	ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
//	for (auto it(triangulation.edges_begin()), it_end(triangulation.edges_end()); it != it_end; ++it)
//	{
//		typename Triangulation::Edge& e = *it;
//
//#ifdef	USE_COLOR_FROM_VERTEX_LABEL_GRADIENT_MESH
//		if (triangulation.face_data[it->first->info()].label
//			!= triangulation.face_data[it->first->neighbor(it->second)->info()].label)
//			draw_svg_line_(ofs, id++, "red", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 2, 1.0);
//#else
//		auto data_e = triangulation.edge_data[Mon_edge(it->first->vertex((it->second + 1) % 3)->info(), it->first->vertex((it->second + 2) % 3)->info())];
//		//const int i = triangulation.label_on_edge(e);
//		switch (data_e.label)
//		{
//		case 0:
//			break;
//		case 1:
//			draw_svg_line_(ofs, id++, "red", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 2, 1.0);
//			break;
//		case 2:
//			draw_svg_line_(ofs, id++, "green", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 4);
//			break;
//		case 3:
//			draw_svg_line_(ofs, id++, "orange", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 6);
//			break;
//		case 4:
//			draw_svg_line_(ofs, id++, "cyan", e.first->vertex((e.second + 1) % 3)->point(), e.first->vertex((e.second + 2) % 3)->point(), 8);
//			break;
//		}
//#endif
//	}
//	if (node)
//		for (auto it(triangulation.vertices_begin()), it_end(triangulation.vertices_end()); it != it_end; ++it)
//			draw_svg_circle_(ofs, id++, "darkgray", it->point(), 3, 1);
//	ofs << "</svg>" << std::endl;
//	ofs.close();
//}

//template<typename Triangulation>
//inline void save_feature_triangle(gl_widget* gl_wgt, const char* file_name, const Triangulation& triangulation, const int rows, const int cols)
//{
//	if (ini_data_cost_img::mode_label_color != mode_color_label::FACE_LABEL_CST)
//		return;
//	cv::Mat_<cv::Vec3f> color_image(rows, cols, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	cv::Mat_<cv::Vec3f> color_image_line(rows, cols, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	cv::Mat_<cv::Vec3f> color_image_line_bw(rows, cols, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	cv::Mat_<cv::Vec3f> bin_class(rows, cols, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	cv::Mat_<cv::Vec3f> bin_class_v2(rows, cols, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	cv::Mat_<cv::Vec3f> classification(rows, cols, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	cv::Mat_<cv::Vec3f> var_triangle(rows, cols, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	cv::Mat_<float> feature((int)triangulation.number_of_faces(), 3, 0.0);
//	float* ptr_feature = feature[0];
//	cv::Mat_<cv::Vec3f> bgr_scalar(1, 1, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	cv::Mat_<cv::Vec3f> lab_scalar(1, 1, cv::Vec3f(0.0f, 0.0f, 0.0f));
//	for (auto it = triangulation.finite_faces_begin(), it_end = triangulation.finite_faces_end(); it != it_end; ++it)
//	{
//		typename Triangulation::Face f = *it;
//		const typename Triangulation::Vertex_handle v1 = f.vertex(0);
//		const typename Triangulation::Vertex_handle v2 = f.vertex(1);
//		const typename Triangulation::Vertex_handle v3 = f.vertex(2);
//		const auto& p1 = v1->point();
//		const auto& p2 = v2->point();
//		const auto& p3 = v3->point();
//	//	std::cout << f.info() << " " << triangulation.face_data.size() <<  std::endl;
//		data data_f = triangulation.face_data[f.info()];
//		const glm::vec3& cfcst = data_f.color;
//		const auto dir = p2 - p1;
//		const auto dir1 = p3 - p1;
//		const auto dir2 = p3 - p2;
//		const glm::vec2 dirgl = glm::vec2(dir.x(), dir.y());
//		const glm::vec2 dirgl1 = glm::vec2(dir1.x(), dir1.y());
//		//	glm::mat2 inv_coord = glm::inverse(glm::outerProduct(dirgl, dirgl1));
//		glm::mat2 inv_coord = glm::inverse(glm::mat2(dirgl, dirgl1));
//		const auto dir_norm = dir / sqrt(dir.squared_length());
//		const auto& normal = dir_norm.perpendicular(CGAL::CLOCKWISE);
//		const auto dir1norm = dir1 / sqrt(dir1.squared_length());
//		const auto dir2norm = dir2 / sqrt(dir2.squared_length());
//		float u_fidelity = 0.0f;
//		int nb_pt = 0;
//		int beta_end = (int)sqrt(CGAL::squared_distance(CGAL::Line_2<typename Triangulation::Vertex_handle::value_type::Geom_traits>(p1, dir), p3));
//		//build features ... like variance inside triangle ... TODO
//		double var = 0.0;
//		for (int beta = 0; beta < beta_end; ++beta)
//		{
//			auto tmp_p1 = p1 - beta / (normal * dir1norm) * dir1norm;
//			auto tmp_p2 = p2 - beta / (normal * dir2norm) * dir2norm;
//			int dst = (int)sqrt((tmp_p2 - tmp_p1).squared_length());
//			auto tmp_pt = tmp_p1;
//			for (int i = 0; i < dst; ++i)
//			{
//				const int x = (int)tmp_pt.x();
//				const int y = (int)tmp_pt.y();
//				if (ini_data_cost_img::is_in_img(x, y))
//				{
//					//dump output image
//					cv::Vec3f& output_color = color_image[y][x];
//					output_color[0] = cfcst.r;
//					output_color[1] = cfcst.g;
//					output_color[2] = cfcst.b;
//
//					++nb_pt;
//					//compute feature
//					const cv::Vec3f& col_cv_tmp = ini_data_cost_img::img_color[y][x];
//					const glm::vec3 colcv = glm::vec3(col_cv_tmp[0], col_cv_tmp[1], col_cv_tmp[2]);
//					var += sqrt(glm::distance2(cfcst, colcv));
//	//				std::cout << cfcst.r << std::endl;
//				}
//				tmp_pt = tmp_pt + dir_norm;
//			}
//		}
//		bgr_scalar[0][0][0] = cfcst[0];
//		bgr_scalar[0][0][1] = cfcst[1];
//		bgr_scalar[0][0][2] = cfcst[2];
//		cv::cvtColor(bgr_scalar, lab_scalar, cv::COLOR_BGR2Lab);
//		*ptr_feature++ = lab_scalar[0][0][0];
//		*ptr_feature++ = lab_scalar[0][0][1];
//		*ptr_feature++ = lab_scalar[0][0][2];
//	//	*ptr_feature++ = (float)(var / nb_pt);
//	}
//
//
//	cv::Mat_<int> label(feature.rows, 1, 0);
//	std::vector<glm::vec3> label_to_color;
//	label_to_color.resize(8);
//	label_to_color[0] = glm::vec3(0.0f, 0.0f, 0.0f);
//	label_to_color[1] = glm::vec3(1.0f, 1.0f, 1.0f);
//	label_to_color[2] = glm::vec3(1.0f, 0.0f, 0.0f);
//	label_to_color[3] = glm::vec3(0.0f, 1.0f, 0.0f);
//	label_to_color[4] = glm::vec3(0.0f, 0.0f, 1.0f);
//	label_to_color[5] = glm::vec3(1.0f, 1.0f, 0.0f);
//	label_to_color[6] = glm::vec3(1.0f, 0.0f, 1.0f);
//	label_to_color[7] = glm::vec3(1.0f, 1.0f, 0.0f);
//	std::cout << "start classif" << std::endl;
//	cv::kmeans(feature, 8, label, cv::TermCriteria(cv::TermCriteria::EPS + cv::TermCriteria::COUNT, 10, 1.0), 3, cv::KMEANS_PP_CENTERS);
//	std::cout << "end classif" << std::endl;
//	//ptr_color = classification[0];
//	int* ptr_label = label[0];
//	ptr_feature = feature[0];
//	//int* img_pixel_label_ptr = img_pixel_label[0];
//	for (auto it = triangulation.finite_faces_begin(), it_end = triangulation.finite_faces_end(); it != it_end; ++it)
//	{
//		typename Triangulation::Face f = *it;
//		const typename Triangulation::Vertex_handle v1 = f.vertex(0);
//		const typename Triangulation::Vertex_handle v2 = f.vertex(1);
//		const typename Triangulation::Vertex_handle v3 = f.vertex(2);
//		const auto& p1 = v1->point();
//		const auto& p2 = v2->point();
//		const auto& p3 = v3->point();
//		data data_f = triangulation.face_data[f.info()];
//		const glm::vec3& cfcst = data_f.color;
//		const auto dir = p2 - p1;
//		const auto dir1 = p3 - p1;
//		const auto dir2 = p3 - p2;
//		const glm::vec2 dirgl = glm::vec2(dir.x(), dir.y());
//		const glm::vec2 dirgl1 = glm::vec2(dir1.x(), dir1.y());
//		//	glm::mat2 inv_coord = glm::inverse(glm::outerProduct(dirgl, dirgl1));
//		glm::mat2 inv_coord = glm::inverse(glm::mat2(dirgl, dirgl1));
//		const auto dir_norm = dir / sqrt(dir.squared_length());
//		const auto& normal = dir_norm.perpendicular(CGAL::CLOCKWISE);
//		const auto dir1norm = dir1 / sqrt(dir1.squared_length());
//		const auto dir2norm = dir2 / sqrt(dir2.squared_length());
//		float u_fidelity = 0.0f;
//		int nb_pt = 0;
//		int beta_end = (int)sqrt(CGAL::squared_distance(CGAL::Line_2<typename Triangulation::Vertex_handle::value_type::Geom_traits>(p1, dir), p3));
//		//build features ... like variance inside triangle ... TODO
//
//	//	std::cout << *ptr_label << std::endl;
//		const glm::vec3& output_color_new = label_to_color[*ptr_label++];
//
//		for (int beta = 0; beta < beta_end; ++beta)
//		{
//			auto tmp_p1 = p1 - beta / (normal * dir1norm) * dir1norm;
//			auto tmp_p2 = p2 - beta / (normal * dir2norm) * dir2norm;
//			int dst = (int)sqrt((tmp_p2 - tmp_p1).squared_length());
//			auto tmp_pt = tmp_p1;
//			for (int i = 0; i < dst; ++i)
//			{
//				const int x = (int)tmp_pt.x();
//				const int y = (int)tmp_pt.y();
//				if (ini_data_cost_img::is_in_img(x, y))
//				{
//					//const cv::Vec3f& col_cv_tmp = Ini_data_cost::img_color[y][x];
//					//const glm::vec3 colcv = glm::vec3(col_cv_tmp[0], col_cv_tmp[1], col_cv_tmp[2]);
//					cv::Vec3f& output_color = classification[y][x];
//					output_color[0] = output_color_new.b;
//					output_color[1] = output_color_new.g;
//					output_color[2] = output_color_new.r;
//
//	//				cv::Vec3f& output_color2 = var_triangle[y][x];
//	//				output_color2[0] = ptr_feature[3];
//	//				output_color2[1] = ptr_feature[3];
//	//				output_color2[2] = ptr_feature[3];
//					
//				}
//				tmp_pt = tmp_pt + dir_norm;
//			}
//		}
////		ptr_feature += 4;
//	}
//	cv::imwrite("classif_tri_8.png", 255.0f * classification);
////	cv::imwrite("classif_var_triangle.png", 255.0f * var_triangle);
//
//
//	std::set<glm::vec3, less_vec3> color_label_1;
//	std::set<glm::vec3, less_vec3> color_label_2;
//	Triangulation::Locate_type lt;
//	int li;
//
//	for (const glm::ivec2& point : ini_data_cost_img::pixel_label_1)
//	{
//		Triangulation::Face_handle fh = triangulation.locate(Triangulation::Point(point.x, point.y), lt, li);
//		color_label_1.emplace(triangulation.face_data[fh->info()].color);
//		//if (lt == Triangulation::VERTEX)
//	}
//	for (const glm::ivec2& point : ini_data_cost_img::pixel_label_2)
//	{
//		Triangulation::Face_handle fh = triangulation.locate(Triangulation::Point(point.x, point.y), lt, li);
//		color_label_2.emplace(triangulation.face_data[fh->info()].color);
//		//if (lt == Triangulation::VERTEX)
//	}
//
//	std::map<typename Triangulation::Face*, int> facet_to_class;
//
//	for (auto it = triangulation.finite_faces_begin(), it_end = triangulation.finite_faces_end(); it != it_end; ++it)
//	{
//		data data_f = triangulation.face_data[it->info()];
//		const glm::vec3& cfcst = data_f.color;
//		int classe = 0;
//		double dst = 1000, tmp_dst;
//		for (const glm::vec3& v : color_label_1)
//		{
//			if ((tmp_dst = glm::distance2(v, cfcst)) < dst)
//				dst = tmp_dst;
//		}
//		for (const glm::vec3& v : color_label_2)
//		{
//			if ((tmp_dst = glm::distance2(v, cfcst)) < dst)
//			{
//				dst = tmp_dst;
//				classe = 1;
//			}
//		}
//		facet_to_class.emplace(&*it, classe);
//	}
//	cv::Vec3f* ptr_color_image = color_image[0];
//	cv::Vec3f* ptr_bin_class = bin_class[0];
//
//	cv::Vec3f* ptr_bin_class_v2 = bin_class_v2[0];
//
//	for (int row = 0; row < rows; ++row)
//	{
//		for (int col = 0; col < cols; ++col)
//		{
//			Triangulation::Face_handle fh = triangulation.locate(Triangulation::Point(col, row), lt, li);
//			glm::vec3& color = triangulation.face_data[fh->info()].color;
//			const int& label = triangulation.face_data[fh->info()].label;
//			int classe = facet_to_class.find(&*fh)->second;
//			cv::Vec3f& output_class = *ptr_bin_class++;
//			cv::Vec3f& output_class_v2 = *ptr_bin_class_v2++;
//			cv::Vec3f& output_color = *ptr_color_image++;
//			output_class[0] = classe;
//			output_class[1] = classe;
//			output_class[2] = classe;
//			output_color[0] = color.r;
//			output_color[1] = color.g;
//			output_color[2] = color.b;
//			output_class_v2[0] = label;
//			output_class_v2[1] = label;
//			output_class_v2[2] = label;
//		}
//	}
//
//
//	//for (auto it = triangulation.finite_faces_begin(), it_end = triangulation.finite_faces_end(); it != it_end; ++it)
//	//{
//	//	typename Triangulation::Face f = *it;
//	//	const typename Triangulation::Vertex_handle v1 = f.vertex(0);
//	//	const typename Triangulation::Vertex_handle v2 = f.vertex(1);
//	//	const typename Triangulation::Vertex_handle v3 = f.vertex(2);
//	//	const auto& p1 = v1->point();
//	//	const auto& p2 = v2->point();
//	//	const auto& p3 = v3->point();
//	//	//	std::cout << f.info() << " " << triangulation.face_data.size() <<  std::endl;
//	//	data data_f = triangulation.face_data[f.info()];
//	//	const glm::vec3& cfcst = data_f.color;
//	//	int classe = 0;
//	//	double dst = 1000, tmp_dst;
//	//	for (const glm::vec3& v : color_label_1)
//	//	{
//	//		if ((tmp_dst = glm::distance2(v, cfcst)) < dst)
//	//			dst = tmp_dst;
//	//	}
//	//	for (const glm::vec3& v : color_label_2)
//	//	{
//	//		if ((tmp_dst = glm::distance2(v, cfcst)) < dst)
//	//		{
//	//			dst = tmp_dst;
//	//			classe = 1;
//	//		}
//	//	}
//	//
//	//
//	//	const auto dir = p2 - p1;
//	//	const auto dir1 = p3 - p1;
//	//	const auto dir2 = p3 - p2;
//	//	const glm::vec2 dirgl = glm::vec2(dir.x(), dir.y());
//	//	const glm::vec2 dirgl1 = glm::vec2(dir1.x(), dir1.y());
//	//	//	glm::mat2 inv_coord = glm::inverse(glm::outerProduct(dirgl, dirgl1));
//	//	glm::mat2 inv_coord = glm::inverse(glm::mat2(dirgl, dirgl1));
//	//	const auto dir_norm = dir / sqrt(dir.squared_length());
//	//	const auto& normal = dir_norm.perpendicular(CGAL::CLOCKWISE);
//	//	const auto dir1norm = dir1 / sqrt(dir1.squared_length());
//	//	const auto dir2norm = dir2 / sqrt(dir2.squared_length());
//	//	float u_fidelity = 0.0f;
//	//	int nb_pt = 0;
//	//	int beta_end = (int)sqrt(CGAL::squared_distance(CGAL::Line_2<typename Triangulation::Vertex_handle::value_type::Geom_traits>(p1, dir), p3));
//	//	//build features ... like variance inside triangle ... TODO
//	//	double var = 0.0;
//	//	for (int beta = 0; beta < beta_end; ++beta)
//	//	{
//	//		auto tmp_p1 = p1 - beta / (normal * dir1norm) * dir1norm;
//	//		auto tmp_p2 = p2 - beta / (normal * dir2norm) * dir2norm;
//	//		int dst = (int)sqrt((tmp_p2 - tmp_p1).squared_length());
//	//		auto tmp_pt = tmp_p1;
//	//		for (int i = 0; i < dst; ++i)
//	//		{
//	//			const int x = (int)tmp_pt.x();
//	//			const int y = (int)tmp_pt.y();
//	//			if (ini_data_cost_img::is_in_img(x, y))
//	//			{
//	//				//dump output image
//	//				cv::Vec3f& output_color = color_image[y][x];
//	//				output_color[0] = classe;
//	//				output_color[1] = classe;
//	//				output_color[2] = classe;
//	//
//	//			}
//	//			tmp_pt = tmp_pt + dir_norm;
//	//		}
//	//	}
//	//}
//	
//	//cv::imwrite((ini_data_cost_img::outut_base_name + "_class_bin.png").c_str(), bin_class*255.0f);
//	cv::imwrite((ini_data_cost_img::outut_base_name + "_binary_our.png").c_str(), bin_class_v2*255.0f);
//	cv::imwrite((ini_data_cost_img::outut_base_name + "_tri.png").c_str(), color_image*255.0f);
//	std::ofstream ofs(std::string(ini_data_cost_img::outut_base_name + "data_error_f1" + ini_data_cost_img::ROOF_PB_MAP_FROM_SUFFIX + ".data"));
//	ofs << ini_data_cost_img::current_def.tp << " " << ini_data_cost_img::current_def.fp << " " << ini_data_cost_img::current_def.tn << " " << ini_data_cost_img::current_def.fn << std::endl;
//	ofs << ini_data_cost_img::current_def.precision << " " << ini_data_cost_img::current_def.recall << " " << ini_data_cost_img::current_def.f << std::endl;
//	ofs.close();
//
//	color_image_line = ini_data_cost_img::img_color.clone();
//	cv::cvtColor(ini_data_cost_img::img.clone(), color_image_line_bw, CV_GRAY2BGR);
//	for (auto it(triangulation.finite_edges_begin()), it_end(triangulation.finite_edges_end()); it != it_end; ++it)
//	{
//		if (triangulation.face_data[it->first->info()].label
//			!= triangulation.face_data[it->first->neighbor(it->second)->info()].label)
//		{
//			const auto& face = *it->first;
//			const int& idx = it->second;
//			const auto& p1 = face.vertex((idx + 1) % 3)->point();
//			const auto& p2 = face.vertex((idx + 2) % 3)->point();
//			cv::line(color_image_line, cv::Point(p1.x(), p1.y()), cv::Point(p2.x(), p2.y()), cv::Scalar(0.0, 0.0, 1.0), 2);
//			cv::line(color_image_line_bw, cv::Point(p1.x(), p1.y()), cv::Point(p2.x(), p2.y()), cv::Scalar(0.0, 0.0, 1.0), 2);
//		}
//	}
//	cv::imwrite((ini_data_cost_img::outut_base_name + "_color_our.png").c_str(), color_image_line*255.0f);
//}
template<typename Triangulation>
inline void save_graph_line_color_from_vertex(const char* file_name, const Triangulation& triangulation, const int rows, const int cols, bool node = true)
{
	cv::Mat_<cv::Vec3f> color_image(rows, cols, cv::Vec3f(0.0f, 0.0f, 0.0f));
	cv::Mat_<float> error_img(rows, cols, 0.0f);
	cv::Vec3f* ptr_color_image = color_image[0];
	cv::Vec3f* ptr_color_image_ref = Triangulation::Ini_data_cost::img_color[0];
	float* ptr_error = error_img[0];
	Triangulation::Locate_type lt;
	int li;
	for (int row = 0; row < rows; ++row)
	{
		for (int col = 0; col < cols; ++col)
		{
			Triangulation::Face_handle fh = triangulation.locate(Triangulation::Point(col + 0.5, row + 0.5), lt, li);
			glm::vec3 color;
			if (lt == Triangulation::Locate_type::FACE)
			{
				const typename Triangulation::Vertex_handle v1 = fh->vertex(0);
				const typename Triangulation::Vertex_handle v2 = fh->vertex(1);
				const typename Triangulation::Vertex_handle v3 = fh->vertex(2);
				const auto& p1 = v1->point();
				const auto& p2 = v2->point();
				const auto& p3 = v3->point();
				const auto dir21 = p2 - p1;
				const auto dir31 = p3 - p1;
				const glm::vec2 dirgl21 = glm::vec2(dir21.x(), dir21.y());
				const glm::vec2 dirgl31 = glm::vec2(dir31.x(), dir31.y());
				glm::mat2 inv_coord = glm::inverse(glm::mat2(dirgl21, dirgl31));
				glm::vec2 deltaX = glm::vec2((double)col - p1.x() + 0.5, (double)row - p1.y() + 0.5);
				glm::vec2 ab = inv_coord * deltaX;

				//get data attached to the vertices
				const auto& c1 = triangulation.vertex_data.find(v1->info())->second.label;
				const auto& c2 = triangulation.vertex_data.find(v2->info())->second.label;
				const auto& c3 = triangulation.vertex_data.find(v3->info())->second.label;
				color = c1 + ab.x * (c2 - c1) + ab.y * (c3 - c1);
			}
			else if (lt == Triangulation::Locate_type::EDGE)
			{
				const typename Triangulation::Vertex_handle v1 = fh->vertex((li + 1) % 3);
				const typename Triangulation::Vertex_handle v2 = fh->vertex((li + 2) % 3);
				const auto& p1 = v1->point();
				const auto& p2 = v2->point();
				const auto dir21 = p2 - p1;
				const float K = glm::length(glm::vec2((double)col - p1.x() + 0.5, (double)row - p1.y() + 0.5))/glm::length(glm::vec2(dir21.x(), dir21.y()));
				const auto& c1 = triangulation.vertex_data.find(v1->info())->second.label;
				const auto& c2 = triangulation.vertex_data.find(v2->info())->second.label;
				color = c1 + K * (c2 - c1);
			}
			else if (lt == Triangulation::Locate_type::VERTEX)
			{
				color = triangulation.vertex_data.find(fh->vertex(li)->info())->second.label;
			}

			cv::Vec3f& output_color = *ptr_color_image++;
			cv::Vec3f& ref_color = *ptr_color_image_ref++;
			const glm::vec3 ref_color_glm = glm::vec3(ref_color[0], ref_color[1], ref_color[2]);
			output_color[0] = color.r;
			output_color[1] = color.g;
			output_color[2] = color.b;
			*ptr_error++ = glm::distance(color, ref_color_glm);
		}
	}
	cv::imwrite(file_name, color_image*255.0f);
	cv::imwrite((Triangulation::Ini_data_cost::outut_base_name + "_vert_error.png").c_str(), error_img*255.0f/sqrt(3.0f));
}

#endif // !_HEADER_SAVE_SVG_FILE___
