#include "texture_opencv.h"
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>

Texture_opencv::Texture_opencv() :base_texture()
{
}


Texture_opencv::Texture_opencv(const cv::Mat_<float> &src) : base_texture()
{
	loadTexture(src);
}
Texture_opencv::Texture_opencv(const cv::Mat_<cv::Vec3f> &src) : base_texture()
{
	loadTexture(src);
}
Texture_opencv::Texture_opencv(const std::string& path) : base_texture()
{
	loadTexture(path);
}

Texture_opencv::Texture_opencv(const GLuint id) : base_texture(id)
{
}

Texture_opencv::~Texture_opencv()
{
}
Texture_opencv::Texture_opencv(const glm::vec2& size) :base_texture(size)
{
}

bool Texture_opencv::save_texture(const std::string& name, unsigned char* data, const int& w, const int& h)
{
	return save_data(name,data,w,h);
}
bool Texture_opencv::save_data(const std::string& name, unsigned char* data, const int& w, const int& h)
{
	cv::Mat_<cv::Vec4b> dst(h, w);
	const int w4 = w << 2;
	unsigned char* ptr_dst = dst.data + (dst.dataend - dst.data) - w4;
	for (int row = 0; row != h; ++row)
	{
		memcpy(ptr_dst, data, w4);
		ptr_dst -= w4;
		data += w4;
	}
	std::vector<int> compression_params;
	compression_params.push_back(CV_IMWRITE_PNG_COMPRESSION);
	compression_params.push_back(0);
	cv::imwrite(name, dst, compression_params);
	return true;
}
bool Texture_opencv::save_data_mono(const std::string& name, unsigned char* data, const int& w, const int& h)
{
	cv::Mat_<unsigned char> dst(h, w);
	unsigned char* ptr_dst = dst.data + (dst.dataend - dst.data) - w;
	for (int row = 0; row != h; ++row)
	{
		memcpy(ptr_dst, data, w);
		ptr_dst -= w;
		data += w;
	}
//while (ptr_dst != ptr_dst_end)
//{
//	unsigned char tmp = *data++;
//	ptr_dst[0] = tmp;
//	ptr_dst[1] = tmp;
//	ptr_dst[2] = tmp;
//	ptr_dst += 3;
//}
	
	std::vector<int> compression_params;
	compression_params.push_back(CV_IMWRITE_PNG_COMPRESSION);
	compression_params.push_back(0);
	cv::imwrite(name, dst, compression_params);
	return true;
}
bool Texture_opencv::loadTexture(const std::string &path)
{
	mName = path;
	
	std::cout << path << std::endl;
	cv::Mat src = cv::imread(path, CV_LOAD_IMAGE_COLOR);
	cv::Mat_<cv::Vec4b> dst(src.rows, src.cols);
	unsigned char* ptr_dst(dst.data);
	unsigned char* ptr_src(src.data);
	const unsigned char* ptr_src_end(src.dataend);
	while (ptr_src != ptr_src_end)
	{
		ptr_dst[0] = ptr_src[2];
		ptr_dst[1] = ptr_src[1];
		ptr_dst[2] = ptr_src[0];
		ptr_dst[3] = 255;
		ptr_dst += 4;
		ptr_src += 3;
	}
	return base_texture::loadTexture(dst.data, dst.cols, dst.rows, path, GL_RGBA);
}
bool Texture_opencv::loadTexture(const cv::Mat_<float> &src)
{
	cv::Mat_<cv::Vec4b> dst(src.rows, src.cols);
	unsigned char* ptr_dst(dst.data);
	float* ptr_src((float*)src.data);
	float* ptr_src_end((float*)src.dataend);
	while (ptr_src != ptr_src_end)
	{
		ptr_dst[0] = ptr_dst[1] = ptr_dst[2] = (unsigned char)(*ptr_src * 255);
		ptr_dst[3] = 255;
		ptr_dst += 4;
		ptr_src ++;
	}
	return base_texture::loadTexture(dst.data, dst.cols, dst.rows, "lol", GL_RGBA);
}
void Texture_opencv::update_texture(const cv::Mat_<float> &src)
{
	cv::Mat_<cv::Vec4b> dst(src.rows, src.cols);
	unsigned char* ptr_dst(dst.data);
	float* ptr_src((float*)src.data);
	float* ptr_src_end((float*)src.dataend);
	while (ptr_src != ptr_src_end)
	{
		ptr_dst[0] = ptr_dst[1] = ptr_dst[2] = (unsigned char)(*ptr_src * 255);
		ptr_dst[3] = 255;
		ptr_dst += 4;
		ptr_src++;
	}
	base_texture::uploadTexture(dst.data, dst.cols, dst.rows, GL_RGBA);
}

bool Texture_opencv::loadTexture(const cv::Mat_<cv::Vec3f>& src)
{
	cv::Mat_<cv::Vec4b> dst(src.rows, src.cols);
	unsigned char* ptr_dst(dst.data);
	float* ptr_src((float*)src.data);
	float* ptr_src_end((float*)src.dataend);
	while (ptr_src != ptr_src_end)
	{
		ptr_dst[0] = (unsigned char)ptr_src[2];
		ptr_dst[1] = (unsigned char)ptr_src[1];
		ptr_dst[2] = (unsigned char)ptr_src[0];
		ptr_dst[3] = 255;
		ptr_dst += 4;
		ptr_src += 3;
	}
	return base_texture::loadTexture(dst.data, dst.cols, dst.rows, "lol_2.0", GL_RGBA);
}

void Texture_opencv::update_texture(const cv::Mat_<cv::Vec3f>& src)
{
	cv::Mat_<cv::Vec4b> dst(src.rows, src.cols);
	unsigned char* ptr_dst(dst.data);
	float* ptr_src((float*)src.data);
	float* ptr_src_end((float*)src.dataend);
	while (ptr_src != ptr_src_end)
	{
		ptr_dst[0] = (unsigned char)ptr_src[2];
		ptr_dst[1] = (unsigned char)ptr_src[1];
		ptr_dst[2] = (unsigned char)ptr_src[0];
		ptr_dst[3] = 255;
		ptr_dst += 4;
		ptr_src += 3;
	}
	base_texture::uploadTexture(dst.data, dst.cols, dst.rows, GL_RGBA);
}