
#define _USE_MATH_DEFINES
#include<cmath>
#define GLM_FORCE_RADIANS
#include<glm/glm.hpp>
#include<glm/ext.hpp>
#include "GCoptimization.h"
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Triangulation_euclidean_traits_2.h>
#include <CGAL/Triangulation_vertex_base_with_info_2.h>


#include <CGAL/Delaunay_triangulation_2.h>
#include <CGAL/Voronoi_diagram_2.h>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc.hpp>
#include<fstream>

typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
typedef CGAL::Triangulation_vertex_base_2<Kernel>                             Vb;
#include<vector>
#include<list>




std::string base_path = "C:\\dev\\project3\\paper\\result\\cmp_graphcut\\horse4\\";
void douglas_peucker(std::list<std::list<glm::vec2>>& lst_pt, const int n);
void save_dp(std::string file_name, std::list<std::list<glm::vec2>>& lst_pt, int rows, int cols);
struct INFOR_V
{
	unsigned int index;
	std::vector<Kernel::Point_2> Vcell;
	Kernel::Point_2 BoundedBox_lt;
	Kernel::Point_2 BoundedBox_rb;
	Kernel::Point_2 p_centoid;	//centroid	
	float b_height;
	float b_ori_height;
	float b_ground;
	int obj_mark;
	Kernel::Point_3 color;
	bool colored;//visited
	std::vector<int> neighbourInd; // index of neighbours, DT index initialized previously 
	std::vector<int> neighbIndLINE;//neighbors based on line-segment region
	int closest_line_seg_ind;
	bool touched_line_seg;
};
typedef CGAL::Triangulation_vertex_base_with_info_2<INFOR_V, Kernel>     D_Vb;
typedef CGAL::Triangulation_face_base_2 <Kernel>                              Fb;
typedef CGAL::Triangulation_data_structure_2<D_Vb,Fb>                         Tds;
typedef CGAL::Delaunay_triangulation_2<Kernel,Tds>						  Triangulation_D;
typedef Triangulation_D::Edge_iterator  Edge_iterator;
typedef Triangulation_D::Vertex_handle Vertex_handle;
typedef Triangulation_D::Vertex_iterator Vertex_iterator;
inline void draw_svg_circle_(std::ofstream& ofs, int id, const std::string& color, const glm::vec2& c, const float& r, const float& stroke_width, const double opacity = 1)
{
	ofs << "<circle ";
	if (id != -1)
		ofs << "id = \"circle_" << id << "\" ";
	ofs << "cx=\"" << c.x << "\" cy=\"" << c.y << "\" r=\"" << r << "\"";
	ofs << " stroke=\"" << color << "\" fill=\"" << color << "\" stroke-width=\"" << stroke_width << "\" opacity=\"" << opacity << "\"/>" << std::endl;
}
inline void draw_svg_line_(std::ofstream& ofs, int id, const std::string& color, const glm::vec2& p1, const glm::vec2& p2, const int wigth = 2, const double opacity = 1)
{
	ofs << "<path ";
	if (id != -1)
		ofs << "id = \"curve_" << id << "\" ";
	ofs << "d = \"M";
	ofs << p1.x << "," << p1.y << " L";
	ofs << p2.x << "," << p2.y << "\"";
	ofs << " stroke=\"" << color << "\" stroke-width=\"" << wigth << "\" fill=\"none\" stroke-opacity=\"" << opacity << "\"/>" << std::endl;
}
inline void draw_svg_line_(std::ofstream& ofs, int id, const std::string& color, const double& p1x, const double& p1y, const double& p2x,const double& p2y, const int wigth = 2, const double opacity = 1)
{
	ofs << "<path ";
	if (id != -1)
		ofs << "id = \"curve_" << id << "\" ";
	ofs << "d = \"M";
	ofs << p1x << "," << p1y << " L";
	ofs << p2x << "," << p2y << "\"";
	ofs << " stroke=\"" << color << "\" stroke-width=\"" << wigth << "\" fill=\"none\" stroke-opacity=\"" << opacity << "\"/>" << std::endl;
}
inline void draw_svg_circle_(std::ofstream& ofs, int id, const std::string& color, const double& cx, const double& cy, const float& r, const float& stroke_width, const double opacity = 1)
{
	ofs << "<circle ";
	if (id != -1)
		ofs << "id = \"circle_" << id << "\" ";
	ofs << "cx=\"" << cx << "\" cy=\"" << cy << "\" r=\"" << r << "\"";
	ofs << " stroke=\"" << color << "\" fill=\"" << color << "\" stroke-width=\"" << stroke_width << "\" opacity=\"" << opacity << "\"/>" << std::endl;
}
void graph_cut(Triangulation_D & T)
{

	cv::Mat input_pb_map = cv::imread(base_path + "bin_horse_proba_map.png");
	std::map<Triangulation_D::Vertex*, std::pair<double,int>> pb_vertex;
	const int rows = input_pb_map.rows;
	const int cols = input_pb_map.cols;
	cv::Mat_<float> out(rows, cols, 0.0f);
	cv::Mat_<float> out2(rows, cols, 0.0f);
	cv::Mat_<float> out3(rows, cols, 0.0f);
	Triangulation_D::Vertex* ptr;
	for (int row = 0; row < rows; ++row)
	{
		for (int col = 0; col < cols; ++col)
		{
			double dst = rows*cols;
			double tmp;
			const double pb = input_pb_map.ptr<cv::Vec3b>(row)[col][0] * 1.0 / 255;
			for (Vertex_iterator vi = T.finite_vertices_begin(); vi != T.finite_vertices_end(); vi++)
			{
				tmp = (vi->point() - Triangulation_D::Point(col, row)).squared_length();
				if (tmp < dst)
				{
					dst = tmp;
					ptr = &*vi;
				}
			}
			auto it = pb_vertex.find(ptr);
			if (it == pb_vertex.end())
			{
				pb_vertex.emplace(ptr,std::pair<double,int>(pb,1));
			}
			else
			{
				it->second.first += pb;
				++(it->second.second);
			}
		}
	}
	for (int row = 0; row < rows; ++row)
	{
		for (int col = 0; col < cols; ++col)
		{
			double dst = rows*cols;
			double tmp;
			for (Vertex_iterator vi = T.finite_vertices_begin(); vi != T.finite_vertices_end(); vi++)
			{
				tmp = (vi->point() - Triangulation_D::Point(col, row)).squared_length();
				if (tmp < dst)
				{
					dst = tmp;
					ptr = &*vi;
				}
			}
			auto it = pb_vertex.find(ptr);
			out[row][col] = it->second.first / it->second.second;
			out2[row][col] = out[row][col]>0.5f?1.0f:0.0f;
		}
	}
	cv::imwrite(base_path + "p1.png", out*255.0f);
	cv::imwrite(base_path + "p2.png", out2*255.0f);

	std::map<Triangulation_D::Vertex*, int> cgal_to_vertex;


		int *data = new int[T.number_of_vertices()*2];
		int *smooth = new int[4];
		for (int l1 = 0; l1 < 2; l1++)
			for (int l2 = 0; l2 < 2; l2++)
				smooth[l1 + l2*2] = (l1 - l2)*(l1 - l2) <= 4 ? (l1 - l2)*(l1 - l2) : 4;
		int i=0;
		for (Vertex_iterator vi = T.finite_vertices_begin(); vi != T.finite_vertices_end(); vi++, i++)
		{
			cgal_to_vertex.emplace(&*vi, i);
			auto it = pb_vertex.find(&*vi);
			int u_data = (int)(255.0*it->second.first / it->second.second);
			for (int l = 0; l < 2; l++)
				if (l == 0) data[i*2 + l] = u_data;
				else data[i*2 + l] = 255- u_data;
		}
		std::map<Triangulation_D::Face*, Triangulation_D::Point> face_to_dump;
		std::map<Triangulation_D::Face*, std::set<Triangulation_D::Face*>> face_adjacency;
		try {
			GCoptimizationGeneralGraph *gc = new GCoptimizationGeneralGraph(T.number_of_vertices(), 2);
			gc->setDataCost(data);
			gc->setSmoothCost(smooth);

			for (Edge_iterator ve = T.finite_edges_begin(); ve != T.finite_edges_end(); ve++)
			{

				int id1 = cgal_to_vertex[&*ve->first->vertex((ve->second + 1) % 3)];
				int id2 = cgal_to_vertex[&*ve->first->vertex((ve->second + 2) % 3)];
				CGAL::Object o = T.dual(ve);
				Triangulation_D::Segment seg;
				Kernel::Ray_2 ray;
				if (assign(seg, o))
				{
					gc->setNeighbors(id1, id2, sqrt(seg.squared_length()));
				}
				else
				{
					gc->setNeighbors(id1, id2, 10000);
				}
			}
			printf("\nBefore optimization energy is %d", gc->compute_energy());
			gc->expansion(2);// run expansion for 2 iterations. For swap use gc->swap(num_iterations);
			//gc->expansion(2);// run expansion for 2 iterations. For swap use gc->swap(num_iterations);
			printf("\nAfter optimization energy is %d", gc->compute_energy());

			for (int row = 0; row < rows; ++row)
			{
				for (int col = 0; col < cols; ++col)
				{
					double dst = rows*cols;
					double tmp;
					for (Vertex_iterator vi = T.finite_vertices_begin(); vi != T.finite_vertices_end(); vi++)
					{
						tmp = (vi->point() - Triangulation_D::Point(col, row)).squared_length();
						if (tmp < dst)
						{
							dst = tmp;
							ptr = &*vi;
						}
					}
					out3[row][col] = gc->whatLabel(cgal_to_vertex[ptr]);
				}
			}
			cv::imwrite(base_path + "p3.png", out3*255.0f);
			gc->expansion(4);// run expansion for 2 iterations. For swap use gc->swap(num_iterations);
							 //gc->expansion(2);// run expansion for 2 iterations. For swap use gc->swap(num_iterations);
			printf("\nAfter optimization energy is %d", gc->compute_energy());

			int id = 0;
			std::ofstream ofs(base_path + "p3.svg");
			ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
			
			face_to_dump.clear();
			for (Edge_iterator ve = T.finite_edges_begin(); ve != T.finite_edges_end(); ve++)
			{
				Triangulation_D::Vertex* ptr1 = &*ve->first->vertex((ve->second + 1) % 3);
				Triangulation_D::Vertex* ptr2 = &*ve->first->vertex((ve->second + 2) % 3);
				int id1 = cgal_to_vertex[ptr1];
				int id2 = cgal_to_vertex[ptr2];
				if (gc->whatLabel(id1) != gc->whatLabel(id2))
				{
					CGAL::Object o = T.dual(ve);
					Triangulation_D::Segment seg;
					if (assign(seg, o))
					{
						draw_svg_line_(ofs, id++, "red", seg.source().x(), seg.source().y(), seg.target().x(), seg.target().y(), 2);
						face_to_dump.emplace(&*ve->first, T.dual(ve->first));
						face_to_dump.emplace(&*ve->first->neighbor(ve->second), T.dual(ve->first->neighbor(ve->second)));
					}
				}
			}
			for (const std::pair< Triangulation_D::Face*, Triangulation_D::Point>& f : face_to_dump)
			{
				Triangulation_D::Point pt = f.second;
				draw_svg_circle_(ofs, id++, "red", pt.x(),pt.y(), 2, 0);
			}
			ofs << "</svg>" << std::endl;
			ofs.close();

			ofs.open(base_path + "p3_overlap.svg");
			ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
			ofs << "<image xlink:href=\"bin_horse_input.jpg" << "\" x=\"0\" y=\"0\" height=\"" << rows << "px\" width=\"" << cols << "px\"/>\n";
			face_to_dump.clear();
			for (Edge_iterator ve = T.finite_edges_begin(); ve != T.finite_edges_end(); ve++)
			{
				Triangulation_D::Vertex* ptr1 = &*ve->first->vertex((ve->second + 1) % 3);
				Triangulation_D::Vertex* ptr2 = &*ve->first->vertex((ve->second + 2) % 3);
				int id1 = cgal_to_vertex[ptr1];
				int id2 = cgal_to_vertex[ptr2];
				if (gc->whatLabel(id1) != gc->whatLabel(id2))
				{
					CGAL::Object o = T.dual(ve);
					Triangulation_D::Segment seg;
					if (assign(seg, o))
					{
						draw_svg_line_(ofs, id++, "red", seg.source().x(), seg.source().y(), seg.target().x(), seg.target().y(), 2);
						face_to_dump.emplace(&*ve->first, T.dual(ve->first));
						face_to_dump.emplace(&*ve->first->neighbor(ve->second), T.dual(ve->first->neighbor(ve->second)));
						auto it = face_adjacency.find(&*ve->first);
						if(it==face_adjacency.end())
						{
							face_adjacency.emplace(&*ve->first, std::set<Triangulation_D::Face*>());
						}
						face_adjacency[&*ve->first].emplace(&*ve->first->neighbor(ve->second));
						it=face_adjacency.find(&*ve->first->neighbor(ve->second));
						if (it == face_adjacency.end())
						{
							face_adjacency.emplace(&*ve->first->neighbor(ve->second), std::set<Triangulation_D::Face*>());
						}
						face_adjacency[&*ve->first->neighbor(ve->second)].emplace(&*ve->first);
					}
				}
			}
			for (const std::pair< Triangulation_D::Face*, Triangulation_D::Point>& f : face_to_dump)
			{
				Triangulation_D::Point pt = f.second;
				draw_svg_circle_(ofs, id++, "red", pt.x(), pt.y(), 2, 0);
			}
			ofs << "</svg>" << std::endl;
			ofs.close();
			for (int row = 0; row < rows; ++row)
			{
				for (int col = 0; col < cols; ++col)
				{
					double dst = rows*cols;
					double tmp;
					for (Vertex_iterator vi = T.finite_vertices_begin(); vi != T.finite_vertices_end(); vi++)
					{
						tmp = (vi->point() - Triangulation_D::Point(col, row)).squared_length();
						if (tmp < dst)
						{
							dst = tmp;
							ptr = &*vi;
						}
					}
					out3[row][col] = gc->whatLabel(cgal_to_vertex[ptr]);
				}
			}
			cv::imwrite(base_path + "p4.png", out3*255.0f);

		std::cout << std::endl << "doublas peucker pre ini step" << std::endl;
		std::list<std::list<glm::vec2>> lst_pt;
		//std::set<Triangulation_D::Face*> lst_face_done;
		std::map<Triangulation_D::Face*, Triangulation_D::Face*> lst_face_done_connection;
		Triangulation_D::Face* previous_face = nullptr;
		for (const std::pair< Triangulation_D::Face*, Triangulation_D::Point>& f : face_to_dump)
		{
		//	std::cout << "base " << lst_face_done_connection.size() << " " << face_to_dump .size()<< std::endl;
			auto it_done = lst_face_done_connection.find(f.first);
			if (it_done != lst_face_done_connection.end())
			{
				//if(it_done->second == prev)
				continue;

			}
			lst_pt.emplace_back(std::list<glm::vec2>());
			std::list<glm::vec2>& lst = lst_pt.back();
			Triangulation_D::Point pt_ini = f.second;
			lst.emplace_back(pt_ini.x(), pt_ini.y());
			previous_face = nullptr;
			Triangulation_D::Face* current_face = f.first;
			Triangulation_D::Face* next_face;
			do {
			//	std::cout << lst_face_done_connection.size() << " " << face_to_dump.size() << std::endl;
				next_face = nullptr;
				int saved_i;
				for (int i = 0; i < 3; ++i)
				{
					Triangulation_D::Face* fptr = &*current_face->neighbor(i);
					if (face_to_dump.find(fptr) != face_to_dump.end())
					{
						//candidate found
						//check for reverse
						if (fptr == previous_face)
							continue;
						//check for forbidden direction (to prevent loop problem ...)
						auto it = lst_face_done_connection.find(current_face);
						if (it != lst_face_done_connection.end())
						{
							if (it->second == fptr)
								continue;
						}
						it = lst_face_done_connection.find(fptr);
						if (it != lst_face_done_connection.end())
						{
							if (it->second == current_face)
								continue;
						}
						auto it2 = face_adjacency.find(current_face);
						if (it2 == face_adjacency.end())
							continue;
						auto it3 = it2->second.find(fptr);
						if (it3 == it2->second.end())
							continue;
						next_face = fptr;
						saved_i = i;
						break;
					}
				}
				if (next_face != nullptr)
				{
					Triangulation_D::Point pt = T.dual(current_face->neighbor(saved_i));
					lst.emplace_back(pt.x(), pt.y());
					lst_face_done_connection.emplace(next_face, current_face);
					previous_face = current_face;
					current_face = next_face;
				}
			} while (next_face != nullptr);
		}
		std::list<std::list<glm::vec2>> copy = lst_pt;
		std::cout << "doublas peucker" << std::endl;
		douglas_peucker(lst_pt, 151);
		save_dp(base_path + "p3_dp_151_h1.svg", lst_pt, rows, cols);
		douglas_peucker(lst_pt, 100);
		save_dp(base_path + "p3_dp_100.svg", lst_pt, rows, cols);
		lst_pt.clear();
		lst_pt = copy;
		douglas_peucker(lst_pt, 50);
		save_dp(base_path + "p3_dp_50.svg", lst_pt, rows, cols);
		lst_pt.clear();
		lst_pt = copy;
		douglas_peucker(lst_pt, 10);
		save_dp(base_path + "p3_dp_10.svg", lst_pt, rows, cols);
		lst_pt.clear();
		lst_pt = copy;
		douglas_peucker(lst_pt, 151);
		save_dp(base_path + "p3_dp_151_h1.svg", lst_pt, rows, cols);
		lst_pt.clear();
		lst_pt = copy;
		douglas_peucker(lst_pt, 132);
		save_dp(base_path + "p3_dp_132_h2.svg", lst_pt, rows, cols);
		lst_pt.clear();
		lst_pt = copy;
		douglas_peucker(lst_pt, 126);
		save_dp(base_path + "p3_dp_126_h4.svg", lst_pt, rows, cols);

		delete gc;

		}
		catch (GCException e) {
			e.Report();
		}

		delete[] data;
		delete[] smooth;
}
void save_dp(std::string file_name, std::list<std::list<glm::vec2>>& lst_pt, int rows, int cols)
{
	int id = 0;
	std::ofstream ofs(file_name);
	ofs << "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"" << rows << "\" width=\"" << cols << "\">" << std::endl;
	ofs << "<image xlink:href=\"bin_horse_input.jpg" << "\" x=\"0\" y=\"0\" height=\"" << rows << "px\" width=\"" << cols << "px\"/>\n";
	for (std::list<glm::vec2>& lst : lst_pt)
		draw_svg_line_(ofs, id++, "red", lst.front(), lst.back(), 2);
	for (std::list<glm::vec2>& lst : lst_pt)
		draw_svg_circle_(ofs, id++, "red", lst.front(), 2, 0);
	ofs << "</svg>" << std::endl;
	ofs.close();
}


void douglas_peucker(std::list<std::list<glm::vec2>>& lst_pt, const int n)
{
	while (lst_pt.size() != n)
	{
		//std::cout << lst_pt.size() << std::endl;
		double dst = 0, tmp;
		std::list<std::list<glm::vec2>>::iterator base_it;
		std::list<glm::vec2>::iterator base_sub_it;
		for (std::list<std::list<glm::vec2>>::iterator it(std::begin(lst_pt)), it_end(std::end(lst_pt)); it != it_end; ++it)
		{
			glm::vec2 p1 = it->front();
			glm::vec2 p2 = it->back();
			if (p1 == p2)
			{
				for (std::list<glm::vec2>::iterator sub_it(++std::begin(*it)), sub_it_end(--std::end(*it)); sub_it != sub_it_end; ++sub_it)
				{
					tmp = glm::length2(*sub_it - p1);
					if (tmp > dst)
					{
						dst = tmp;
						base_sub_it = sub_it;
						base_it = it;
					}
				}
			}
			else
			{
				glm::vec2 dir = glm::normalize(p2 - p1);
				for (std::list<glm::vec2>::iterator sub_it(++std::begin(*it)), sub_it_end(--std::end(*it)); sub_it != sub_it_end; ++sub_it)
				{
					glm::vec2 MP = *sub_it - p1;
					tmp = glm::length2(MP) - std::pow(glm::dot(dir, MP), 2);
					if (tmp > dst)
					{
						dst = tmp;
						base_sub_it = sub_it;
						base_it = it;
					}
				}
			}
		}
		if (dst == 0)
			break;
		std::list<std::list<glm::vec2>>::iterator new_lst = lst_pt.insert(base_it, std::list<glm::vec2>());
		new_lst->splice(new_lst->begin(), *base_it, base_it->begin(), base_sub_it);
		new_lst->emplace_back(*base_sub_it);//appear in the two list because we take the distance to the line defining by the first and the last point of the sub list
	}
}